@@ -4,12 +4,13 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: config.c 1.134 2005/06/18 10:30:02 kls Exp $
+ * $Id: config.c 1.138 2005/09/09 15:08:59 kls Exp $
  */
 
 #include "config.h"
 #include <ctype.h>
 #include <stdlib.h>
+#include "device.h"
 #include "i18n.h"
 #include "interface.h"
 #include "plugin.h"
@@ -266,7 +267,7 @@ cSetup::cSetup(void)
   AudioLanguages[0] = -1;
   EPGLanguages[0] = -1;
   EPGScanTimeout = 5;
-  EPGBugfixLevel = 2;
+  EPGBugfixLevel = 3;
   EPGLinger = 0;
   SVDRPTimeout = 300;
   ZapTimeout = 3;
@@ -295,7 +296,7 @@ cSetup::cSetup(void)
   MaxVideoFileSize = MAXVIDEOFILESIZE;
   SplitEditedFiles = 0;
   MinEventTimeout = 30;
-  MinUserInactivity = 120;
+  MinUserInactivity = 300;
   MultiSpeedMode = 0;
   ShowReplayMode = 0;
   ResumeID = 0;
@@ -472,6 +473,7 @@ bool cSetup::Save(void)
   Store("OSDTheme",           OSDTheme);
   Store("PrimaryDVB",         PrimaryDVB);
   Store("ShowInfoOnChSwitch", ShowInfoOnChSwitch);
+  Store("MenuScrollPage",     MenuScrollPage);
   Store("MenuScrollWrap",     MenuScrollWrap);
   Store("MarkInstantRecord",  MarkInstantRecord);
   Store("NameInstantRecord",  NameInstantRecord);
