@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: cutter.c 1.8 2005/05/15 14:21:08 kls Exp $
+ * $Id: cutter.c 1.11 2005/10/31 12:26:44 kls Exp $
  */
 
 #include "cutter.h"
@@ -18,8 +18,7 @@
 class cCuttingThread : public cThread {
 private:
   const char *error;
-  bool active;
-  int fromFile, toFile;
+  cUnbufferedFile *fromFile, *toFile;
   cFileName *fromFileName, *toFileName;
   cIndexFile *fromIndex, *toIndex;
   cMarks fromMarks, toMarks;
@@ -35,8 +34,7 @@ cCuttingThread::cCuttingThread(const char *FromFileName, const char *ToFileName)
 :cThread("video cutting")
 {
   error = NULL;
-  active = false;
-  fromFile = toFile = -1;
+  fromFile = toFile = NULL;
   fromFileName = toFileName = NULL;
   fromIndex = toIndex = NULL;
   if (fromMarks.Load(FromFileName) && fromMarks.Count()) {
@@ -53,7 +51,6 @@ cCuttingThread::cCuttingThread(const char *FromFileName, const char *ToFileName)
 
 cCuttingThread::~cCuttingThread()
 {
-  active = false;
   Cancel(3);
   delete fromFileName;
   delete toFileName;
@@ -67,7 +64,8 @@ void cCuttingThread::Action(void)
   if (Mark) {
      fromFile = fromFileName->Open();
      toFile = toFileName->Open();
-     active = fromFile >= 0 && toFile >= 0;
+     if (!fromFile || !toFile)
+        return;
      int Index = Mark->position;
      Mark = fromMarks.Next(Mark);
      int FileSize = 0;
@@ -78,7 +76,7 @@ void cCuttingThread::Action(void)
      uchar buffer[MAXFRAMESIZE];
      bool LastMark = false;
      bool cutIn = true;
-     while (active) {
+     while (Running()) {
            uchar FileNumber;
            int FileOffset, Length;
            uchar PictureType;
@@ -94,7 +92,7 @@ void cCuttingThread::Action(void)
                  fromFile = fromFileName->SetOffset(FileNumber, FileOffset);
                  CurrentFileNumber = FileNumber;
                  }
-              if (fromFile >= 0) {
+              if (fromFile) {
                  int len = ReadFrame(fromFile, buffer,  Length, sizeof(buffer));
                  if (len < 0) {
                     error = "ReadFrame";
@@ -133,7 +131,7 @@ void cCuttingThread::Action(void)
                  cutIn = false;
                  }
               }
-           if (safe_write(toFile, buffer, Length) < 0) {
+           if (toFile->Write(buffer, Length) < 0) {
               error = "safe_write";
               break;
               }
