@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: device.c 1.103 2005/06/12 13:39:11 kls Exp $
+ * $Id: device.c 1.112 2005/11/26 12:56:09 kls Exp $
  */
 
 #include "device.h"
@@ -139,7 +139,6 @@ int cPesAssembler::PacketSize(const uchar *data)
 
 // The default priority for non-primary devices:
 #define DEFAULTPRIORITY  -1
-#define TUNER_LOCK_TIMEOUT 5000 // ms
 
 int cDevice::numDevices = 0;
 int cDevice::useDevice = 0;
@@ -156,8 +155,6 @@ cDevice::cDevice(void)
 
   SetVideoFormat(Setup.VideoFormat);
 
-  active = false;
-
   mute = false;
   volume = Setup.CurrentVolume;
 
@@ -197,6 +194,20 @@ cDevice::~cDevice()
   delete pesAssembler;
 }
 
+bool cDevice::WaitForAllDevicesReady(int Timeout)
+{
+  for (time_t t0 = time(NULL); time(NULL) - t0 < Timeout; ) {
+      bool ready = true;
+      for (int i = 0; i < numDevices; i++) {
+          if (device[i] && !device[i]->Ready())
+             ready = false;
+          }
+      if (ready)
+         return true;
+      }
+  return false;
+}
+
 void cDevice::SetUseDevice(int n)
 {
   if (n < MAXDEVICES)
@@ -299,22 +310,6 @@ cDevice *cDevice::GetDevice(const cChannel *Channel, int Priority, bool *NeedsDe
             }
          }
       }
-
-  /*XXX+ too complex with multiple recordings per device
-  if (!d && Ca > MAXDEVICES) {
-     // We didn't find one the easy way, so now we have to try harder:
-     int ShiftLevel = -1;
-     for (int i = 0; i < numDevices; i++) {
-         if (Provides[i]) { // this device is basicly able to do the job, but for some reason we didn't get it above
-            int sl = device[i]->CanShift(Ca, Priority); // asks this device to shift its job to another device
-            if (sl >= 0 && (ShiftLevel < 0 || sl < ShiftLevel)) {
-               d = device[i]; // found one that can be shifted with the fewest number of subsequent shifts
-               ShiftLevel = sl;
-               }
-            }
-         }
-     }
-  XXX*/
   return d;
 }
 
@@ -402,6 +397,8 @@ bool cDevice::AddPid(int Pid, ePidType PidType)
               DelPid(Pid, PidType);
               return false;
               }
+           if (ciHandler)
+              ciHandler->SetPid(Pid, true);
            }
         PRINTPIDS("a");
         return true;
@@ -429,6 +426,8 @@ bool cDevice::AddPid(int Pid, ePidType PidType)
            DelPid(Pid, PidType);
            return false;
            }
+        if (ciHandler)
+           ciHandler->SetPid(Pid, true);
         }
      }
   return true;
@@ -455,6 +454,8 @@ void cDevice::DelPid(int Pid, ePidType PidType)
            if (pidHandles[n].used == 0) {
               pidHandles[n].handle = -1;
               pidHandles[n].pid = 0;
+              if (ciHandler)
+                 ciHandler->SetPid(Pid, false);
               }
            }
         PRINTPIDS("E");
@@ -519,6 +520,11 @@ bool cDevice::ProvidesChannel(const cChannel *Channel, int Priority, bool *Needs
   return false;
 }
 
+bool cDevice::MaySwitchTransponder(void)
+{
+  return !Receiving(true) && !(pidHandles[ptAudio].pid || pidHandles[ptVideo].pid || pidHandles[ptDolby].pid);
+}
+
 bool cDevice::SwitchChannel(const cChannel *Channel, bool LiveView)
 {
   if (LiveView)
@@ -601,12 +607,34 @@ eSetChannelResult cDevice::SetChannel(const cChannel *Channel, bool LiveView)
         sectionHandler->SetStatus(false);
         sectionHandler->SetChannel(NULL);
         }
+     // Tell the ciHandler about the channel switch and add all PIDs of this
+     // channel to it, for possible later decryption:
+     if (ciHandler) {
+        ciHandler->SetSource(Channel->Source(), Channel->Transponder());
+// Men at work - please stand clear! ;-)
+#ifdef XXX_DO_MULTIPLE_CA_CHANNELS
+        if (Channel->Ca() > CACONFBASE) {
+#endif
+           ciHandler->AddPid(Channel->Sid(), Channel->Vpid(), 2);
+           for (const int *Apid = Channel->Apids(); *Apid; Apid++)
+               ciHandler->AddPid(Channel->Sid(), *Apid, 4);
+           for (const int *Dpid = Channel->Dpids(); *Dpid; Dpid++)
+               ciHandler->AddPid(Channel->Sid(), *Dpid, 0);
+#ifdef XXX_DO_MULTIPLE_CA_CHANNELS
+           bool CanDecrypt = ciHandler->CanDecrypt(Channel->Sid());//XXX
+           dsyslog("CanDecrypt %d %d %d %s", CardIndex() + 1, CanDecrypt, Channel->Number(), Channel->Name());//XXX
+           }
+#endif
+        }
      if (SetChannelDevice(Channel, LiveView)) {
         // Start section handling:
         if (sectionHandler) {
            sectionHandler->SetChannel(Channel);
            sectionHandler->SetStatus(true);
            }
+        // Start decrypting any PIDs the might have been set in SetChannelDevice():
+        if (ciHandler)
+           ciHandler->StartDecrypting();
         }
      else
         Result = scrFailed;
@@ -784,13 +812,17 @@ void cDevice::EnsureAudioTrack(bool Force)
 {
   if (Force || !availableTracks[currentAudioTrack].id) {
      eTrackType PreferredTrack = ttAudioFirst;
+     int PreferredAudioChannel = 0;
      int LanguagePreference = -1;
      int StartCheck = Setup.CurrentDolby ? ttDolbyFirst : ttAudioFirst;
      int EndCheck = ttDolbyLast;
      for (int i = StartCheck; i <= EndCheck; i++) {
          const tTrackId *TrackId = GetTrack(eTrackType(i));
-         if (TrackId && TrackId->id && I18nIsPreferredLanguage(Setup.AudioLanguages, I18nLanguageIndex(TrackId->language), LanguagePreference))
+         int pos = 0;
+         if (TrackId && TrackId->id && I18nIsPreferredLanguage(Setup.AudioLanguages, TrackId->language, LanguagePreference, &pos)) {
             PreferredTrack = eTrackType(i);
+            PreferredAudioChannel = pos;
+            }
          if (Setup.CurrentDolby && i == ttDolbyLast) {
             i = ttAudioFirst - 1;
             EndCheck = ttAudioLast;
@@ -800,8 +832,9 @@ void cDevice::EnsureAudioTrack(bool Force)
      const tTrackId *Track = GetTrack(GetCurrentAudioTrack());
      if (Force || !Track || !Track->id || PreferredTrack != GetCurrentAudioTrack()) {
         if (!Force) // only log this for automatic changes
-           dsyslog("setting audio track to %d", PreferredTrack);
+           dsyslog("setting audio track to %d (%d)", PreferredTrack, PreferredAudioChannel);
         SetCurrentAudioTrack(PreferredTrack);
+        SetAudioChannel(PreferredAudioChannel);
         }
      }
 }
@@ -1068,41 +1101,9 @@ int cDevice::Priority(void) const
   return priority;
 }
 
-int cDevice::CanShift(int Ca, int Priority, int UsedCards) const
+bool cDevice::Ready(void)
 {
-  return -1;//XXX+ too complex with multiple recordings per device
-  /*XXX
-  // Test whether a receiver on this device can be shifted to another one
-  // in order to perform a new receiving with the given Ca and Priority on this device:
-  int ShiftLevel = -1; // default means this device can't be shifted
-  if (UsedCards & (1 << CardIndex()) != 0)
-     return ShiftLevel; // otherwise we would get into a loop
-  if (Receiving()) {
-     if (ProvidesCa(Ca) // this device provides the requested Ca
-        && (Ca != this->Ca() // the requested Ca is different from the one currently used...
-           || Priority > this->Priority())) { // ...or the request comes from a higher priority
-        cDevice *d = NULL;
-        int Provides[MAXDEVICES];
-        UsedCards |= (1 << CardIndex());
-        for (int i = 0; i < numDevices; i++) {
-            if ((Provides[i] = device[i]->ProvidesCa(this->Ca())) != 0) { // this device is basicly able to do the job
-               if (device[i] != this) { // it is not _this_ device
-                  int sl = device[i]->CanShift(this->Ca(), Priority, UsedCards); // this is the original Priority!
-                  if (sl >= 0 && (ShiftLevel < 0 || sl < ShiftLevel)) {
-                     d = device[i];
-                     ShiftLevel = sl;
-                     }
-                  }
-               }
-            }
-        if (ShiftLevel >= 0)
-           ShiftLevel++; // adds the device's own shift
-        }
-     }
-  else if (Priority > this->Priority())
-     ShiftLevel = 0; // no shifting necessary, this device can do the job
-  return ShiftLevel;
-  XXX*/
+  return true;
 }
 
 int cDevice::ProvidesCa(const cChannel *Channel) const
@@ -1126,25 +1127,25 @@ bool cDevice::Receiving(bool CheckAny) const
 
 void cDevice::Action(void)
 {
-  if (active && OpenDvr()) {
-     for (; active;) {
-         // Read data from the DVR device:
-         uchar *b = NULL;
-         if (GetTSPacket(b)) {
-            if (b) {
-               int Pid = (((uint16_t)b[1] & PID_MASK_HI) << 8) | b[2];
-               // Distribute the packet to all attached receivers:
-               Lock();
-               for (int i = 0; i < MAXRECEIVERS; i++) {
-                   if (receiver[i] && receiver[i]->WantsPid(Pid))
-                      receiver[i]->Receive(b, TS_SIZE);
-                   }
-               Unlock();
-               }
-            }
-         else
-            break;
-         }
+  if (Running() && OpenDvr()) {
+     while (Running()) {
+           // Read data from the DVR device:
+           uchar *b = NULL;
+           if (GetTSPacket(b)) {
+              if (b) {
+                 int Pid = (((uint16_t)b[1] & PID_MASK_HI) << 8) | b[2];
+                 // Distribute the packet to all attached receivers:
+                 Lock();
+                 for (int i = 0; i < MAXRECEIVERS; i++) {
+                     if (receiver[i] && receiver[i]->WantsPid(Pid))
+                        receiver[i]->Receive(b, TS_SIZE);
+                     }
+                 Unlock();
+                 }
+              }
+           else
+              break;
+           }
      CloseDvr();
      }
 }
@@ -1169,10 +1170,15 @@ bool cDevice::AttachReceiver(cReceiver *Receiver)
      return false;
   if (Receiver->device == this)
      return true;
+// activate the following line if you need it - actually the driver should be fixed!
+//#define WAIT_FOR_TUNER_LOCK
+#ifdef WAIT_FOR_TUNER_LOCK
+#define TUNER_LOCK_TIMEOUT 5000 // ms
   if (!HasLock(TUNER_LOCK_TIMEOUT)) {
      esyslog("ERROR: device %d has no lock, can't attach receiver!", CardIndex() + 1);
      return false;
      }
+#endif
   cMutexLock MutexLock(&mutexReceiver);
   for (int i = 0; i < MAXRECEIVERS; i++) {
       if (!receiver[i]) {
@@ -1188,10 +1194,10 @@ bool cDevice::AttachReceiver(cReceiver *Receiver)
          Receiver->device = this;
          receiver[i] = Receiver;
          Unlock();
-         if (!active) {
-            active = true;
+         if (!Running())
             Start();
-            }
+         if (ciHandler)
+            ciHandler->StartDecrypting();
          return true;
          }
       }
@@ -1218,10 +1224,10 @@ void cDevice::Detach(cReceiver *Receiver)
       else if (receiver[i])
          receiversLeft = true;
       }
-  if (!receiversLeft) {
-     active = false;
+  if (ciHandler)
+     ciHandler->StartDecrypting();
+  if (!receiversLeft)
      Cancel(3);
-     }
 }
 
 void cDevice::DetachAll(int Pid)
@@ -1246,13 +1252,11 @@ cTSBuffer::cTSBuffer(int File, int Size, int CardIndex)
   delivered = false;
   ringBuffer = new cRingBufferLinear(Size, TS_SIZE, true, "TS");
   ringBuffer->SetTimeouts(100, 100);
-  active = true;
   Start();
 }
 
 cTSBuffer::~cTSBuffer()
 {
-  active = false;
   Cancel(3);
   delete ringBuffer;
 }
@@ -1262,20 +1266,20 @@ void cTSBuffer::Action(void)
   if (ringBuffer) {
      bool firstRead = true;
      cPoller Poller(f);
-     for (; active;) {
-         if (firstRead || Poller.Poll(100)) {
-            firstRead = false;
-            int r = ringBuffer->Read(f);
-            if (r < 0 && FATALERRNO) {
-               if (errno == EOVERFLOW)
-                  esyslog("ERROR: driver buffer overflow on device %d", cardIndex);
-               else {
-                  LOG_ERROR;
-                  break;
-                  }
-               }
-            }
-         }
+     while (Running()) {
+           if (firstRead || Poller.Poll(100)) {
+              firstRead = false;
+              int r = ringBuffer->Read(f);
+              if (r < 0 && FATALERRNO) {
+                 if (errno == EOVERFLOW)
+                    esyslog("ERROR: driver buffer overflow on device %d", cardIndex);
+                 else {
+                    LOG_ERROR;
+                    break;
+                    }
+                 }
+              }
+           }
      }
 }
 
