@@ -4,12 +4,13 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: device.h 1.59 2005/06/12 13:35:47 kls Exp $
+ * $Id: device.h 1.66 2005/11/05 15:25:41 kls Exp $
  */
 
 #ifndef __DEVICE_H
 #define __DEVICE_H
 
+#include "channels.h"
 #include "ci.h"
 #include "eit.h"
 #include "filter.h"
@@ -81,12 +82,11 @@ enum eTrackType { ttNone,
 #define IS_DOLBY_TRACK(t) (ttDolbyFirst <= (t) && (t) <= ttDolbyLast)
 
 struct tTrackId {
-  uint16_t id;          // The PES packet id or the PID.
-  char language[8];     // something like either "eng" or "deu/eng"
-  char description[32]; // something like "Dolby Digital 5.1"
+  uint16_t id;                  // The PES packet id or the PID.
+  char language[MAXLANGCODE2];  // something like either "eng" or "deu+eng"
+  char description[32];         // something like "Dolby Digital 5.1"
   };
 
-class cChannel;
 class cPlayer;
 class cReceiver;
 class cPesAssembler;
@@ -102,6 +102,12 @@ private:
 public:
   static int NumDevices(void) { return numDevices; }
          ///< Returns the total number of devices.
+  static bool WaitForAllDevicesReady(int Timeout = 0);
+         ///< Waits until all devices have become ready, or the given Timeout
+         ///< (seconds) has expired. While waiting, the Ready() function of each
+         ///< device is called in turn, until they all return true.
+         ///< \return True if all devices have become ready within the given
+         ///< timeout.
   static void SetUseDevice(int n);
          ///< Sets the 'useDevice' flag of the given device.
          ///< If this function is not called before initializing, all devices
@@ -136,6 +142,11 @@ private:
 protected:
   cDevice(void);
   virtual ~cDevice();
+  virtual bool Ready(void);
+         ///< Returns true if this device is ready. Devices with conditional
+         ///< access hardware may need some time until they are up and running.
+         ///< This function is called in a loop at startup until all devices
+         ///< are ready (see WaitForAllDevicesReady()).
   static int NextCardIndex(int n = 0);
          ///< Calculates the next card index.
          ///< Each device in a given machine must have a unique card index, which
@@ -204,6 +215,9 @@ public:
          ///< function itself actually returns true.
          ///< The default implementation always returns false, so a derived cDevice
          ///< class that can provide channels must implement this function.
+  virtual bool MaySwitchTransponder(void);
+         ///< Returns true if it is ok to switch the transponder on this device,
+         ///< without disturbing any other activities.
   bool SwitchChannel(const cChannel *Channel, bool LiveView);
          ///< Switches the device to the given Channel, initiating transfer mode
          ///< if necessary.
@@ -233,7 +247,6 @@ public:
 // PID handle facilities
 
 private:
-  bool active;
   virtual void Action(void);
 protected:
   enum ePidType { ptAudio, ptVideo, ptPcr, ptTeletext, ptDolby, ptOther };
@@ -256,7 +269,7 @@ protected:
          ///< On indicates whether the PID shall be added or deleted.
          ///< Handle->handle can be used by the device to store information it
          ///< needs to receive this PID (for instance a file handle).
-         ///< Handle->used indicated how many receivers are using this PID.
+         ///< Handle->used indicates how many receivers are using this PID.
          ///< Type indicates some special types of PIDs, which the device may
          ///< need to set in a specific way.
 
@@ -444,8 +457,7 @@ public:
        ///< Returns true if the device itself or any of the file handles in
        ///< Poller is ready for further action.
        ///< If TimeoutMs is not zero, the device will wait up to the given number
-       ///< of milleseconds before returning in case there is no immediate
-       ///< need for data.
+       ///< of milleseconds before returning in case it can't accept any data.
   virtual bool Flush(int TimeoutMs = 0);
        ///< Returns true if the device's output buffers are empty, i. e. any
        ///< data which was bufferd so far has been processed.
@@ -476,7 +488,6 @@ public:
 private:
   cMutex mutexReceiver;
   cReceiver *receiver[MAXRECEIVERS];
-  int CanShift(int Ca, int Priority, int UsedCards = 0) const;
 protected:
   int Priority(void) const;
       ///< Returns the priority of the current receiving session (0..MAXPRIORITY),
@@ -519,7 +530,6 @@ class cTSBuffer : public cThread {
 private:
   int f;
   int cardIndex;
-  bool active;
   bool delivered;
   cRingBufferLinear *ringBuffer;
   virtual void Action(void);
