@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: dvbdevice.c 1.131 2005/06/19 11:00:43 kls Exp $
+ * $Id: dvbdevice.c 1.138 2005/11/26 13:23:11 kls Exp $
  */
 
 #include "dvbdevice.h"
@@ -35,6 +35,7 @@ extern "C" {
 
 #define DO_REC_AND_PLAY_ON_PRIMARY_DEVICE 1
 #define DO_MULTIPLE_RECORDINGS 1
+//#define DO_MULTIPLE_CA_CHANNELS
 
 #define DEV_VIDEO         "/dev/video"
 #define DEV_DVB_ADAPTER   "/dev/dvb/adapter"
@@ -69,20 +70,17 @@ static int DvbOpen(const char *Name, int n, int Mode, bool ReportError = false)
 
 class cDvbTuner : public cThread {
 private:
-  enum eTunerStatus { tsIdle, tsSet, tsTuned, tsLocked, tsCam };
+  enum eTunerStatus { tsIdle, tsSet, tsTuned, tsLocked };
   int fd_frontend;
   int cardIndex;
   fe_type_t frontendType;
   cCiHandler *ciHandler;
   cChannel channel;
   const char *diseqcCommands;
-  bool active;
-  bool useCa;
-  time_t startTime;
   eTunerStatus tunerStatus;
   cMutex mutex;
   cCondVar locked;
-  cCondWait newSet;
+  cCondVar newSet;
   bool GetFrontendEvent(dvb_frontend_event &Event, int TimeoutMs = 0);
   bool SetFrontend(void);
   virtual void Action(void);
@@ -90,7 +88,7 @@ public:
   cDvbTuner(int Fd_Frontend, int CardIndex, fe_type_t FrontendType, cCiHandler *CiHandler);
   virtual ~cDvbTuner();
   bool IsTunedTo(const cChannel *Channel) const;
-  void Set(const cChannel *Channel, bool Tune, bool UseCa);
+  void Set(const cChannel *Channel, bool Tune);
   bool Locked(int TimeoutMs = 0);
   };
 
@@ -101,10 +99,7 @@ cDvbTuner::cDvbTuner(int Fd_Frontend, int CardIndex, fe_type_t FrontendType, cCi
   frontendType = FrontendType;
   ciHandler = CiHandler;
   diseqcCommands = NULL;
-  active = false;
-  useCa = false;
   tunerStatus = tsIdle;
-  startTime = time(NULL);
   if (frontendType == FE_QPSK)
      CHECK(ioctl(fd_frontend, FE_SET_VOLTAGE, SEC_VOLTAGE_13)); // must explicitly turn on LNB power
   SetDescription("tuner on device %d", cardIndex + 1);
@@ -113,9 +108,9 @@ cDvbTuner::cDvbTuner(int Fd_Frontend, int CardIndex, fe_type_t FrontendType, cCi
 
 cDvbTuner::~cDvbTuner()
 {
-  active = false;
   tunerStatus = tsIdle;
-  newSet.Signal();
+  newSet.Broadcast();
+  locked.Broadcast();
   Cancel(3);
 }
 
@@ -124,23 +119,21 @@ bool cDvbTuner::IsTunedTo(const cChannel *Channel) const
   return tunerStatus != tsIdle && channel.Source() == Channel->Source() && channel.Transponder() == Channel->Transponder();
 }
 
-void cDvbTuner::Set(const cChannel *Channel, bool Tune, bool UseCa)
+void cDvbTuner::Set(const cChannel *Channel, bool Tune)
 {
-  Lock();
+  cMutexLock MutexLock(&mutex);
   if (Tune)
      tunerStatus = tsSet;
-  else if (tunerStatus == tsCam)
-     tunerStatus = tsLocked;
-  useCa = UseCa;
-  if (Channel->Ca() && tunerStatus != tsCam)
-     startTime = time(NULL);
   channel = *Channel;
-  Unlock();
-  newSet.Signal();
+  newSet.Broadcast();
 }
 
 bool cDvbTuner::Locked(int TimeoutMs)
 {
+  bool isLocked = (tunerStatus >= tsLocked);
+  if (isLocked || !TimeoutMs)
+     return isLocked;
+
   cMutexLock MutexLock(&mutex);
   if (TimeoutMs && tunerStatus < tsLocked)
      locked.TimedWait(mutex, TimeoutMs);
@@ -294,52 +287,37 @@ bool cDvbTuner::SetFrontend(void)
 void cDvbTuner::Action(void)
 {
   dvb_frontend_event event;
-  active = true;
-  while (active) {
-        Lock();
-        if (tunerStatus == tsSet) {
-           while (GetFrontendEvent(event))
-                 ; // discard stale events
-           tunerStatus = SetFrontend() ? tsTuned : tsIdle;
-           }
-        if (tunerStatus != tsIdle) {
-           while (GetFrontendEvent(event, 10)) {
-                 if (event.status & FE_REINIT) {
-                    tunerStatus = tsSet;
-                    esyslog("ERROR: frontend %d was reinitialized - re-tuning", cardIndex);
-                    }
-                 if (event.status & FE_HAS_LOCK) {
-                    cMutexLock MutexLock(&mutex);
-                    tunerStatus = tsLocked;
-                    locked.Broadcast();
-                    }
-                 }
-           }
-        if (ciHandler) {
-           if (ciHandler->Process() && useCa) {
-              if (tunerStatus == tsLocked) {
-                 for (int Slot = 0; Slot < ciHandler->NumSlots(); Slot++) {
-                     cCiCaPmt CaPmt(channel.Source(), channel.Transponder(), channel.Sid(), ciHandler->GetCaSystemIds(Slot));
-                     if (CaPmt.Valid()) {
-                        CaPmt.AddPid(channel.Vpid(), 2);
-                        CaPmt.AddPid(channel.Apid(0), 4);
-                        CaPmt.AddPid(channel.Apid(1), 4);
-                        CaPmt.AddPid(channel.Dpid(0), 0);
-                        if (ciHandler->SetCaPmt(CaPmt, Slot)) {
-                           tunerStatus = tsCam;
-                           startTime = 0;
-                           }
-                        }
+  while (Running()) {
+        bool hasEvent = GetFrontendEvent(event, 1);
+
+        cMutexLock MutexLock(&mutex);
+        switch (tunerStatus) {
+          case tsIdle:
+               break;
+          case tsSet:
+               if (hasEvent)
+                  continue;
+               tunerStatus = SetFrontend() ? tsTuned : tsIdle;
+               continue;
+          case tsTuned:
+          case tsLocked:
+               if (hasEvent) {
+                  if (event.status & FE_REINIT) {
+                     tunerStatus = tsSet;
+                     esyslog("ERROR: frontend %d was reinitialized - re-tuning", cardIndex);
                      }
-                 }
-              }
-           else if (tunerStatus > tsLocked)
-              tunerStatus = tsLocked;
-           }
-        Unlock();
-        // in the beginning we loop more often to let the CAM connection start up fast
+                  if (event.status & FE_HAS_LOCK) {
+                     tunerStatus = tsLocked;
+                     locked.Broadcast();
+                     }
+                  continue;
+                  }
+          }
+
+        if (ciHandler)
+           ciHandler->Process();
         if (tunerStatus != tsTuned)
-           newSet.Wait((ciHandler && (time(NULL) - startTime < 20)) ? 100 : 1000);
+           newSet.TimedWait(mutex, 1000);
         }
 }
 
@@ -482,6 +460,15 @@ bool cDvbDevice::HasDecoder(void) const
   return fd_video >= 0 && fd_audio >= 0;
 }
 
+bool cDvbDevice::Ready(void)
+{
+  if (ciHandler) {
+     ciHandler->Process();
+     return ciHandler->Ready();
+     }
+  return true;
+}
+
 int cDvbDevice::ProvidesCa(const cChannel *Channel) const
 {
   if (Channel->Ca() >= 0x0100 && ciHandler) {
@@ -643,6 +630,11 @@ eVideoSystem cDvbDevice::GetVideoSystem(void)
   return VideoSystem;
 }
 
+bool cDvbDevice::SetAudioBypass(bool On)
+{
+  return ioctl(fd_audio, AUDIO_SET_BYPASS_MODE, On) == 0;
+}
+
 //                            ptAudio        ptVideo        ptPcr        ptTeletext        ptDolby        ptOther
 dmx_pes_type_t PesTypes[] = { DMX_PES_AUDIO, DMX_PES_VIDEO, DMX_PES_PCR, DMX_PES_TELETEXT, DMX_PES_OTHER, DMX_PES_OTHER };
 
@@ -761,9 +753,12 @@ bool cDvbDevice::ProvidesChannel(const cChannel *Channel, int Priority, bool *Ne
         if (dvbTuner->IsTunedTo(Channel)) {
            if (Channel->Vpid() && !HasPid(Channel->Vpid()) || Channel->Apid(0) && !HasPid(Channel->Apid(0))) {
 #ifdef DO_MULTIPLE_RECORDINGS
+#ifndef DO_MULTIPLE_CA_CHANNELS
               if (Ca() > CACONFBASE || Channel->Ca() > CACONFBASE)
                  needsDetachReceivers = Ca() != Channel->Ca();
-              else if (!IsPrimaryDevice())
+              else
+#endif
+              if (!IsPrimaryDevice())
                  result = true;
 #ifdef DO_REC_AND_PLAY_ON_PRIMARY_DEVICE
               else
@@ -813,18 +808,19 @@ bool cDvbDevice::SetChannelDevice(const cChannel *Channel, bool LiveView)
 
   // Set the tuner:
 
-  dvbTuner->Set(Channel, DoTune, !EITScanner.UsesDevice(this)); //XXX 1.3: this is an ugly hack - find a cleaner solution//XXX
+  dvbTuner->Set(Channel, DoTune);
 
   // If this channel switch was requested by the EITScanner we don't wait for
   // a lock and don't set any live PIDs (the EITScanner will wait for the lock
   // by itself before setting any filters):
 
-  if (EITScanner.UsesDevice(this))
+  if (EITScanner.UsesDevice(this)) //XXX
      return true;
 
   // PID settings:
 
   if (TurnOnLivePIDs) {
+     SetAudioBypass(false);
      if (!(AddPid(Channel->Ppid(), ptPcr) && AddPid(Channel->Vpid(), ptVideo) && AddPid(Channel->Apid(0), ptAudio))) {
         esyslog("ERROR: failed to set PIDs for channel %d on device %d", Channel->Number(), CardIndex() + 1);
         return false;
@@ -894,7 +890,8 @@ void cDvbDevice::SetAudioTrackDevice(eTrackType Type)
 {
   const tTrackId *TrackId = GetTrack(Type);
   if (TrackId && TrackId->id) {
-     if (IS_AUDIO_TRACK(Type)) {
+     SetAudioBypass(false);
+     if (IS_AUDIO_TRACK(Type) || (IS_DOLBY_TRACK(Type) && SetAudioBypass(true))) {
         if (pidHandles[ptAudio].pid && pidHandles[ptAudio].pid != TrackId->id) {
            DetachAll(pidHandles[ptAudio].pid);
            pidHandles[ptAudio].pid = TrackId->id;
@@ -1056,22 +1053,6 @@ void cDvbDevice::Mute(void)
 
 void cDvbDevice::StillPicture(const uchar *Data, int Length)
 {
-/* Using the VIDEO_STILLPICTURE ioctl call would be the
-   correct way to display a still frame, but unfortunately this
-   doesn't work with frames from VDR. So let's do pretty much the
-   same here as in DVB/driver/dvb.c's play_iframe() - I have absolutely
-   no idea why it works this way, but doesn't work with VIDEO_STILLPICTURE.
-   If anybody ever finds out what could be changed so that VIDEO_STILLPICTURE
-   could be used, please let me know!
-   kls 2002-03-23
-   2003-08-30: apparently the driver can't handle PES data, so Oliver Endriss
-               <o.endriss@gmx.de> has changed this to strip all PES headers
-               and send pure ES data to the driver. Seems to work just fine!
-               Let's drop the VIDEO_STILLPICTURE_WORKS_WITH_VDR_FRAMES stuff
-               once this has proven to work in all cases.
-*/
-#define VIDEO_STILLPICTURE_WORKS_WITH_VDR_FRAMES
-#ifdef VIDEO_STILLPICTURE_WORKS_WITH_VDR_FRAMES
   if (Data[0] == 0x00 && Data[1] == 0x00 && Data[2] == 0x01 && (Data[3] & 0xF0) == 0xE0) {
      // PES data
      char *buf = MALLOC(char, Length);
@@ -1143,13 +1124,6 @@ void cDvbDevice::StillPicture(const uchar *Data, int Length)
      video_still_picture sp = { (char *)Data, Length };
      CHECK(ioctl(fd_video, VIDEO_STILLPICTURE, &sp));
      }
-#else
-#define MIN_IFRAME 400000
-  for (int i = MIN_IFRAME / Length + 1; i > 0; i--) {
-      safe_write(fd_video, Data, Length);
-      cCondWait::SleepMs(3); // allows the buffer to be displayed in case the progress display is active
-      }
-#endif
 }
 
 bool cDvbDevice::Poll(cPoller &Poller, int TimeoutMs)
