@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: dvbplayer.c 1.34 2005/05/22 11:26:51 kls Exp $
+ * $Id: dvbplayer.c 1.41 2005/10/31 12:33:48 kls Exp $
  */
 
 #include "dvbplayer.h"
@@ -74,12 +74,11 @@ int cBackTrace::Get(bool Forward)
 
 class cNonBlockingFileReader : public cThread {
 private:
-  int f;
+  cUnbufferedFile *f;
   uchar *buffer;
   int wanted;
   int length;
   bool hasData;
-  bool active;
   cCondWait newSet;
 protected:
   void Action(void);
@@ -87,24 +86,22 @@ public:
   cNonBlockingFileReader(void);
   ~cNonBlockingFileReader();
   void Clear(void);
-  int Read(int FileHandle, uchar *Buffer, int Length);
+  int Read(cUnbufferedFile *File, uchar *Buffer, int Length);
   bool Reading(void) { return buffer; }
   };
 
 cNonBlockingFileReader::cNonBlockingFileReader(void)
 :cThread("non blocking file reader")
 {
-  f = -1;
+  f = NULL;
   buffer = NULL;
   wanted = length = 0;
   hasData = false;
-  active = false;
   Start();
 }
 
 cNonBlockingFileReader::~cNonBlockingFileReader()
 {
-  active = false;
   newSet.Signal();
   Cancel(3);
   free(buffer);
@@ -113,7 +110,7 @@ cNonBlockingFileReader::~cNonBlockingFileReader()
 void cNonBlockingFileReader::Clear(void)
 {
   Lock();
-  f = -1;
+  f = NULL;
   free(buffer);
   buffer = NULL;
   wanted = length = 0;
@@ -122,7 +119,7 @@ void cNonBlockingFileReader::Clear(void)
   newSet.Signal();
 }
 
-int cNonBlockingFileReader::Read(int FileHandle, uchar *Buffer, int Length)
+int cNonBlockingFileReader::Read(cUnbufferedFile *File, uchar *Buffer, int Length)
 {
   if (hasData && buffer) {
      if (buffer != Buffer) {
@@ -134,7 +131,7 @@ int cNonBlockingFileReader::Read(int FileHandle, uchar *Buffer, int Length)
      return length;
      }
   if (!buffer) {
-     f = FileHandle;
+     f = File;
      buffer = Buffer;
      wanted = Length;
      length = 0;
@@ -147,11 +144,10 @@ int cNonBlockingFileReader::Read(int FileHandle, uchar *Buffer, int Length)
 
 void cNonBlockingFileReader::Action(void)
 {
-  active = true;
-  while (active) {
+  while (Running()) {
         Lock();
-        if (!hasData && f >= 0 && buffer) {
-           int r = safe_read(f, buffer + length, wanted - length);
+        if (!hasData && f && buffer) {
+           int r = f->Read(buffer + length, wanted - length);
            if (r >= 0) {
               length += r;
               if (!r || length == wanted) // r == 0 means EOF
@@ -185,10 +181,8 @@ private:
   cBackTrace *backTrace;
   cFileName *fileName;
   cIndexFile *index;
-  int replayFile;
+  cUnbufferedFile *replayFile;
   bool eof;
-  bool active;
-  bool running;
   bool firstPacket;
   ePlayModes playMode;
   ePlayDirs playDir;
@@ -207,7 +201,7 @@ protected:
 public:
   cDvbPlayer(const char *FileName);
   virtual ~cDvbPlayer();
-  bool Active(void) { return active; }
+  bool Active(void) { return cThread::Running(); }
   void Pause(void);
   void Play(void);
   void Forward(void);
@@ -233,8 +227,6 @@ cDvbPlayer::cDvbPlayer(const char *FileName)
   backTrace = NULL;
   index = NULL;
   eof = false;
-  active = true;
-  running = false;
   firstPacket = true;
   playMode = pmPlay;
   playDir = pdForward;
@@ -245,7 +237,7 @@ cDvbPlayer::cDvbPlayer(const char *FileName)
   isyslog("replay %s", FileName);
   fileName = new cFileName(FileName, false);
   replayFile = fileName->Open();
-  if (replayFile < 0)
+  if (!replayFile)
      return;
   ringBuffer = new cRingBufferFrame(PLAYERBUFSIZE);
   // Create the index file:
@@ -310,10 +302,10 @@ bool cDvbPlayer::NextFile(uchar FileNumber, int FileOffset)
 {
   if (FileNumber > 0)
      replayFile = fileName->SetOffset(FileNumber, FileOffset);
-  else if (replayFile >= 0 && eof)
+  else if (replayFile && eof)
      replayFile = fileName->NextFile();
   eof = false;
-  return replayFile >= 0;
+  return replayFile != NULL;
 }
 
 int cDvbPlayer::Resume(void)
@@ -350,14 +342,11 @@ bool cDvbPlayer::Save(void)
 void cDvbPlayer::Activate(bool On)
 {
   if (On) {
-     if (replayFile >= 0)
+     if (replayFile)
         Start();
      }
-  else if (active) {
-     running = false;
+  else
      Cancel(9);
-     active = false;
-     }
 }
 
 void cDvbPlayer::Action(void)
@@ -374,8 +363,7 @@ void cDvbPlayer::Action(void)
   int Length = 0;
   bool Sleep = false;
 
-  running = true;
-  while (running && (NextFile() || readIndex >= 0 || ringBuffer->Available() || !DeviceFlush(100))) {
+  while (Running() && (NextFile() || readIndex >= 0 || ringBuffer->Available() || !DeviceFlush(100))) {
         if (Sleep) {
            cCondWait::SleepMs(3); // this keeps the CPU load low
            Sleep = false;
@@ -388,7 +376,7 @@ void cDvbPlayer::Action(void)
            // Read the next frame from the file:
 
            if (playMode != pmStill && playMode != pmPause) {
-              if (!readFrame && (replayFile >= 0 || readIndex >= 0)) {
+              if (!readFrame && (replayFile || readIndex >= 0)) {
                  if (!nonBlockingFileReader->Reading()) {
                     if (playMode == pmFast || (playMode == pmSlow && playDir == pdBackward)) {
                        uchar FileNumber;
@@ -497,9 +485,10 @@ void cDvbPlayer::Action(void)
                  p = NULL;
                  }
               }
+           else
+              Sleep = true;
            }
         }
-  active = running = false;
 
   cNonBlockingFileReader *nbfr = nonBlockingFileReader;
   nonBlockingFileReader = NULL;
@@ -632,7 +621,10 @@ int cDvbPlayer::SkipFrames(int Frames)
      int Current, Total;
      GetIndex(Current, Total, true);
      int OldCurrent = Current;
-     Current = index->GetNextIFrame(Current + Frames, Frames > 0);
+     // As GetNextIFrame() increments/decrements at least once, the 
+     // destination frame (= Current + Frames) must be adjusted by
+     // -1/+1 respectively.
+     Current = index->GetNextIFrame(Current + Frames + (Frames > 0 ? -1 : 1), Frames > 0);
      return Current >= 0 ? Current : OldCurrent;
      }
   return -1;
@@ -666,11 +658,34 @@ void cDvbPlayer::Goto(int Index, bool Still)
      int FileOffset, Length;
      Index = index->GetNextIFrame(Index, false, &FileNumber, &FileOffset, &Length);
      if (Index >= 0 && NextFile(FileNumber, FileOffset) && Still) {
-        uchar b[MAXFRAMESIZE];
+        uchar b[MAXFRAMESIZE + 4 + 5 + 4];
         int r = ReadFrame(replayFile, b, Length, sizeof(b));
         if (r > 0) {
            if (playMode == pmPause)
               DevicePlay();
+           // append sequence end code to get the image shown immediately with softdevices
+           if (r > 6 && (b[3] & 0xF0) == 0xE0) { // make sure to append it only to a video packet
+              b[r++] = 0x00;
+              b[r++] = 0x00;
+              b[r++] = 0x01;
+              b[r++] = b[3];
+              if (b[6] & 0x80) { // MPEG 2
+                 b[r++] = 0x00;
+                 b[r++] = 0x07;
+                 b[r++] = 0x80;
+                 b[r++] = 0x00;
+                 b[r++] = 0x00;
+                 }
+              else { // MPEG 1
+                 b[r++] = 0x00;
+                 b[r++] = 0x05;
+                 b[r++] = 0x0F;
+                 }
+              b[r++] = 0x00;
+              b[r++] = 0x00;
+              b[r++] = 0x01;
+              b[r++] = 0xB7;
+              }
            DeviceStillPicture(b, r);
            }
         playMode = pmStill;
