@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: eitscan.c 1.26 2005/06/12 14:09:45 kls Exp $
+ * $Id: eitscan.c 1.29 2005/11/05 15:24:36 kls Exp $
  */
 
 #include "eitscan.h"
@@ -127,7 +127,7 @@ void cEITScanner::Activity(void)
 
 void cEITScanner::Process(void)
 {
-  if (Setup.EPGScanTimeout && Channels.MaxNumber() > 1) {
+  if ((Setup.EPGScanTimeout || !lastActivity) && Channels.MaxNumber() > 1) { // !lastActivity means a scan was forced
      time_t now = time(NULL);
      if (now - lastScan > ScanTimeout && now - lastActivity > ActivityTimeout) {
         if (Channels.Lock(false, 10)) {
@@ -150,12 +150,14 @@ void cEITScanner::Process(void)
                          if (!Channel->Ca() || Channel->Ca() == Device->DeviceNumber() + 1 || Channel->Ca() >= 0x0100) {
                             if (Device->ProvidesTransponder(Channel)) {
                                if (!Device->Receiving()) {
-                                  if (Device != cDevice::ActualDevice() || (Device->ProvidesTransponderExclusively(Channel) && Setup.EPGScanTimeout && now - lastActivity > Setup.EPGScanTimeout * 3600)) {
-                                     if (Device == cDevice::ActualDevice() && !currentChannel) {
-                                        if (cTransferControl::ReceiverDevice())
+                                  bool MaySwitchTransponder = Device->MaySwitchTransponder();
+                                  if (MaySwitchTransponder || Device->ProvidesTransponderExclusively(Channel) && now - lastActivity > Setup.EPGScanTimeout * 3600) {
+                                     if (!MaySwitchTransponder) {
+                                        if (Device == cDevice::ActualDevice() && !currentChannel) {
                                            cDevice::PrimaryDevice()->StopReplay(); // stop transfer mode
-                                        currentChannel = Device->CurrentChannel();
-                                        Skins.Message(mtInfo, tr("Starting EPG scan"));
+                                           currentChannel = Device->CurrentChannel();
+                                           Skins.Message(mtInfo, tr("Starting EPG scan"));
+                                           }
                                         }
                                      currentDevice = Device;//XXX see also dvbdevice.c!!!
                                      //dsyslog("EIT scan: device %d  source  %-8s tp %5d", Device->DeviceNumber() + 1, *cSource::ToString(Channel->Source()), Channel->Transponder());
