@@ -7,7 +7,7 @@
  * Original version (as used in VDR before 1.3.0) written by
  * Robert Schneider <Robert.Schneider@web.de> and Rolf Hakenes <hakenes@hippomi.de>.
  *
- * $Id: epg.c 1.35 2005/06/05 12:17:15 kls Exp $
+ * $Id: epg.c 1.40 2005/11/11 13:37:43 kls Exp $
  */
 
 #include "epg.h"
@@ -285,8 +285,10 @@ bool cEvent::Read(FILE *f, cSchedule *Schedule)
   if (Schedule) {
      cEvent *Event = NULL;
      char *s;
+     int line = 0;
      cReadLine ReadLine;
      while ((s = ReadLine.Read(f)) != NULL) {
+           line++;
            char *t = skipspace(s + 1);
            switch (*s) {
              case 'E': if (!Event) {
@@ -298,6 +300,8 @@ bool cEvent::Read(FILE *f, cSchedule *Schedule)
                           if (n == 3 || n == 4) {
                              Event = (cEvent *)Schedule->GetEvent(EventID, StartTime);
                              cEvent *newEvent = NULL;
+                             if (Event)
+                                DELETENULL(Event->components);
                              if (!Event)
                                 Event = newEvent = new cEvent(EventID);
                              if (Event) {
@@ -314,8 +318,10 @@ bool cEvent::Read(FILE *f, cSchedule *Schedule)
                        break;
              case 'c': // to keep things simple we react on 'c' here
                        return true;
-             default:  if (Event && !Event->Parse(s))
+             default:  if (Event && !Event->Parse(s)) {
+                          esyslog("ERROR: EPG data problem in line %d", line);
                           return false;
+                          }
              }
            }
      esyslog("ERROR: unexpected end of file while reading EPG data");
@@ -419,32 +425,6 @@ void cEvent::FixEpgBugs(void)
   // EPG data. Let's fix their bugs as good as we can:
   if (title) {
 
-     // Some channels put too much information into the ShortText and leave the
-     // Description empty:
-     //
-     // Title
-     // (NAT, Year Min')[ ["ShortText". ]Description]
-     //
-     if (shortText && !description) {
-        if (*shortText == '(') {
-           char *e = strchr(shortText + 1, ')');
-           if (e) {
-              if (*(e + 1)) {
-                 if (*++e == ' ')
-                    if (*(e + 1) == '"')
-                       e++;
-                 }
-              else
-                 e = NULL;
-              char *s = e ? strdup(e) : NULL;
-              free(shortText);
-              shortText = s;
-              EpgBugFixStat(0, ChannelID());
-              // now the fixes #1 and #2 below will handle the rest
-              }
-           }
-        }
-
      // Some channels put the ShortText in quotes and use either the ShortText
      // or the Description field, depending on how long the string is:
      //
@@ -469,7 +449,7 @@ void cEvent::FixEpgBugs(void)
            }
         }
 
-     // Some channels put the Description into the ShortText (preceeded
+     // Some channels put the Description into the ShortText (preceded
      // by a blank) if there is no actual ShortText and the Description
      // is short enough:
      //
@@ -523,19 +503,6 @@ void cEvent::FixEpgBugs(void)
      title = compactspace(title);
      shortText = compactspace(shortText);
      description = compactspace(description);
-     // Remove superfluous hyphens:
-     if (description) {
-        char *p = description;
-        while (*p && *(p + 1) && *(p + 2)) {
-              if (*p == '-' && *(p + 1) == ' ' && p != description && islower(*(p - 1)) && islower(*(p + 2))) {
-                 if (!startswith(p + 2, "und ")) { // special case in German, as in "Lach- und Sachgeschichten"
-                    memmove(p, p + 2, strlen(p + 2) + 1);
-                    EpgBugFixStat(5, ChannelID());
-                    }
-                 }
-              p++;
-              }
-        }
 
 #define MAX_USEFUL_EPISODE_LENGTH 40
      // Some channels put a whole lot of information in the ShortText and leave
