@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: interface.c 1.68 2004/11/01 14:23:28 kls Exp $
+ * $Id: interface.c 1.70 2005/11/27 15:31:06 kls Exp $
  */
 
 #include "interface.h"
@@ -36,13 +36,6 @@ eKeys cInterface::GetKey(bool Wait)
   if (SVDRP) {
      if (SVDRP->Process())
         Wait = false;
-     if (!Skins.IsOpen()) {
-        char *message = SVDRP->GetMessage();
-        if (message) {
-           Skins.Message(mtInfo, message);
-           free(message);
-           }
-        }
      }
   return cRemote::Get(Wait ? 1000 : 10);
 }
@@ -56,7 +49,7 @@ eKeys cInterface::Wait(int Seconds, bool KeepChar)
   time_t timeout = time(NULL) + Seconds;
   for (;;) {
       Key = GetKey();
-      if ((Key != kNone && (RAWKEY(Key) != kOk || RAWKEY(Key) == Key)) || time(NULL) > timeout || interrupted)
+      if (ISRAWKEY(Key) || time(NULL) > timeout || interrupted)
          break;
       }
   if (KeepChar && ISRAWKEY(Key))
