@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: keys.c 1.7 2004/12/27 11:08:34 kls Exp $
+ * $Id: keys.c 1.9 2005/09/17 11:27:40 kls Exp $
  */
 
 #include "keys.h"
@@ -208,9 +208,8 @@ bool cKeyMacro::Parse(char *s)
                  esyslog("ERROR: @plugin can't be first in macro");
                  return false;
                  }
-              macro[n++] = k_Plugin;
+              macro[n] = k_Plugin;
               if (n < MAXKEYSINMACRO) {
-                 macro[n] = kOk;
                  plugin = strdup(p + 1);
                  if (!cPluginManager::GetPlugin(plugin)) {
                     esyslog("ERROR: unknown plugin '%s'", plugin);
