@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: menu.h 1.70 2005/05/15 14:34:54 kls Exp $
+ * $Id: menu.h 1.77 2005/11/05 17:26:09 kls Exp $
  */
 
 #ifndef __MENU_H
@@ -15,6 +15,7 @@
 #include "epg.h"
 #include "osdbase.h"
 #include "dvbplayer.h"
+#include "menuitems.h"
 #include "recorder.h"
 #include "skins.h"
 
@@ -29,14 +30,36 @@ public:
   virtual eOSState ProcessKey(eKeys Key);
   };
 
+class cMenuEditTimer : public cOsdMenu {
+private:
+  cTimer *timer;
+  cTimer data;
+  int channel;
+  bool addIfConfirmed;
+  cMenuEditDateItem *firstday;
+  void SetFirstDayItem(void);
+public:
+  cMenuEditTimer(cTimer *Timer, bool New = false);
+  virtual ~cMenuEditTimer();
+  virtual eOSState ProcessKey(eKeys Key);
+  };
+
+class cMenuEvent : public cOsdMenu {
+private:
+  const cEvent *event;
+public:
+  cMenuEvent(const cEvent *Event, bool CanSwitch = false);
+  virtual void Display(void);
+  virtual eOSState ProcessKey(eKeys Key);
+  };
+
 class cMenuMain : public cOsdMenu {
 private:
-  time_t lastActivity;
   bool replaying;
   static cOsdObject *pluginOsdObject;
-  void Set(const char *Plugin = NULL);
+  void Set(void);
 public:
-  cMenuMain(bool Replaying, eOSState State = osUnknown, const char *Plugin = NULL);
+  cMenuMain(bool Replaying, eOSState State = osUnknown);
   virtual eOSState ProcessKey(eKeys Key);
   static cOsdObject *PluginOsdObject(void);
   };
@@ -96,8 +119,9 @@ public:
 class cMenuCam : public cOsdMenu {
 private:
   cCiMenu *ciMenu;
-  time_t lastActivity;
   bool selected;
+  int offset;
+  void AddMultiLineItem(const char *s);
   eOSState Select(void);
 public:
   cMenuCam(cCiMenu *CiMenu);
@@ -108,7 +132,6 @@ public:
 class cMenuCamEnquiry : public cOsdMenu {
 private:
   cCiEnquiry *ciEnquiry;
-  time_t lastActivity;
   char *input;
   bool replied;
   eOSState Reply(void);
@@ -126,8 +149,10 @@ class cMenuRecordings : public cOsdMenu {
 private:
   char *base;
   int level;
-  static int helpKeys;
+  int recordingsState;
+  int helpKeys;
   void SetHelpKeys(void);
+  void Set(bool Refresh = false);
   cRecording *GetRecording(cMenuRecordingItem *Item);
   bool Open(bool OpenSubMenus = false);
   eOSState Play(void);
