@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: osdbase.c 1.20 2005/06/18 10:30:51 kls Exp $
+ * $Id: osdbase.c 1.24 2005/10/09 10:56:26 kls Exp $
  */
 
 #include "osdbase.h"
@@ -25,12 +25,12 @@ cOsdItem::cOsdItem(eOSState State)
   fresh = true;
 }
 
-cOsdItem::cOsdItem(const char *Text, eOSState State)
+cOsdItem::cOsdItem(const char *Text, eOSState State, bool Selectable)
 {
   text = NULL;
   offset = -1;
   state = State;
-  selectable = true;
+  selectable = Selectable;
   fresh = true;
   SetText(Text);
 }
@@ -196,8 +196,11 @@ void cOsdMenu::Display(void)
   int count = Count();
   if (count > 0) {
      int ni = 0;
-     for (cOsdItem *item = First(); item; item = Next(item))
+     for (cOsdItem *item = First(); item; item = Next(item)) {
          cStatus::MsgOsdItem(item->Text(), ni++);
+         if (current < 0 && item->Selectable())
+            current = item->Index();
+         }
      if (current < 0)
         current = 0; // just for safety - there HAS to be a current item!
      if (current - first >= displayMenuItems || current < first) {
@@ -210,8 +213,9 @@ void cOsdMenu::Display(void)
      int i = first;
      int n = 0;
      for (cOsdItem *item = Get(first); item; item = Next(item)) {
-         displayMenu->SetItem(item->Text(), i - first, i == current, item->Selectable());
-         if (i == current)
+         bool CurrentSelectable = (i == current) && item->Selectable();
+         displayMenu->SetItem(item->Text(), i - first, CurrentSelectable, item->Selectable());
+         if (CurrentSelectable)
             cStatus::MsgOsdCurrentItem(item->Text());
          if (++n == displayMenuItems)
             break;
@@ -238,8 +242,8 @@ void cOsdMenu::DisplayCurrent(bool Current)
 {
   cOsdItem *item = Get(current);
   if (item) {
-     displayMenu->SetItem(item->Text(), current - first, Current, item->Selectable());
-     if (Current)
+     displayMenu->SetItem(item->Text(), current - first, Current && item->Selectable(), item->Selectable());
+     if (Current && item->Selectable())
         cStatus::MsgOsdCurrentItem(item->Text());
      if (!Current)
         item->SetFresh(true); // leaving the current item resets 'fresh'
@@ -264,14 +268,16 @@ void cOsdMenu::CursorUp(void)
   int tmpCurrent = current;
   int lastOnScreen = first + displayMenuItems - 1;
   int last = Count() - 1;
+  if (last < 0)
+     return;
   while (--tmpCurrent != current) {
         if (tmpCurrent < 0) {
            if (Setup.MenuScrollWrap)
-              tmpCurrent = last;
+              tmpCurrent = last + 1;
            else
               return;
            }
-        if (SelectableItem(tmpCurrent))
+        else if (SelectableItem(tmpCurrent))
            break;
         }
   if (first <= tmpCurrent && tmpCurrent <= lastOnScreen)
@@ -294,14 +300,16 @@ void cOsdMenu::CursorDown(void)
   int tmpCurrent = current;
   int lastOnScreen = first + displayMenuItems - 1;
   int last = Count() - 1;
+  if (last < 0)
+     return;
   while (++tmpCurrent != current) {
         if (tmpCurrent > last) {
            if (Setup.MenuScrollWrap)
-              tmpCurrent = 0;
+              tmpCurrent = -1;
            else
               return;
            }
-        if (SelectableItem(tmpCurrent))
+        else if (SelectableItem(tmpCurrent))
            break;
         }
   if (first <= tmpCurrent && tmpCurrent <= lastOnScreen)
