@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: plugin.h 1.8 2005/01/30 14:03:48 kls Exp $
+ * $Id: plugin.h 1.10 2005/08/27 16:13:17 kls Exp $
  */
 
 #ifndef __PLUGIN_H
@@ -50,6 +50,10 @@ public:
 
   void RegisterI18n(const tI18nPhrase * const Phrases);
 
+  virtual bool Service(const char *Id, void *Data = NULL);
+  virtual const char **SVDRPHelpPages(void);
+  virtual cString SVDRPCommand(const char *Command, const char *Option, int &ReplyCode);
+
   static void SetConfigDirectory(const char *Dir);
   static const char *ConfigDirectory(const char *PluginName = NULL);
   };
@@ -88,6 +92,8 @@ public:
   static bool HasPlugins(void);
   static cPlugin *GetPlugin(int Index);
   static cPlugin *GetPlugin(const char *Name);
+  static cPlugin *CallFirstService(const char *Id, void *Data = NULL);
+  static bool CallAllServices(const char *Id, void *Data = NULL);
   void StopPlugins(void);
   void Shutdown(void);
   };
