@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: recording.c 1.109 2005/06/05 14:11:45 kls Exp $
+ * $Id: recording.c 1.124 2005/11/04 14:19:44 kls Exp $
  */
 
 #include "recording.h"
@@ -36,15 +36,8 @@
 #define DATAFORMAT   "%4d-%02d-%02d.%02d:%02d.%02d.%02d" RECEXT
 #define NAMEFORMAT   "%s/%s/" DATAFORMAT
 */
-// start of implementation for brain dead systems
 #define DATAFORMAT   "%4d-%02d-%02d.%02d%*c%02d.%02d.%02d" RECEXT
-#ifdef VFAT
-#define nameFORMAT   "%4d-%02d-%02d.%02d.%02d.%02d.%02d" RECEXT
-#else
-#define nameFORMAT   "%4d-%02d-%02d.%02d:%02d.%02d.%02d" RECEXT
-#endif
-#define NAMEFORMAT   "%s/%s/" nameFORMAT
-// end of implementation for brain dead systems
+#define NAMEFORMAT   "%s/%s/" "%4d-%02d-%02d.%02d.%02d.%02d.%02d" RECEXT
 
 #define RESUMEFILESUFFIX  "/resume%s%s.vdr"
 #ifdef SUMMARYFALLBACK
@@ -65,17 +58,25 @@
 
 #define MAX_SUBTITLE_LENGTH  40
 
+bool VfatFileSystem = false;
+
+static cRecordings DeletedRecordings(true);
+
 void RemoveDeletedRecordings(void)
 {
   static time_t LastRemoveCheck = 0;
-  if (time(NULL) - LastRemoveCheck > REMOVECHECKDELTA) {
+  if (LastRemoveCheck == 0) {
+     DeletedRecordings.Update();
+     LastRemoveCheck = time(NULL) - REMOVECHECKDELTA * 9 / 10;
+     }
+  else if (time(NULL) - LastRemoveCheck > REMOVECHECKDELTA) {
      // Make sure only one instance of VDR does this:
      cLockFile LockFile(VideoDirectory);
      if (!LockFile.Lock())
         return;
      // Remove the oldest file that has been "deleted":
-     cRecordings DeletedRecordings(true);
-     if (DeletedRecordings.Load()) {
+     cThreadLock DeletedRecordingsLock(&DeletedRecordings);
+     if (DeletedRecordings.Count()) {
         cRecording *r = DeletedRecordings.First();
         cRecording *r0 = r;
         while (r) {
@@ -85,11 +86,14 @@ void RemoveDeletedRecordings(void)
               }
         if (r0 && time(NULL) - r0->start > DELETEDLIFETIME * 3600) {
            r0->Remove();
+           DeletedRecordings.Del(r0);
            RemoveEmptyVideoDirectories();
            LastRemoveCheck += REMOVELATENCY;
            return;
            }
         }
+     else
+        DeletedRecordings.Update();
      LastRemoveCheck = time(NULL);
      }
 }
@@ -109,8 +113,8 @@ void AssertFreeDiskSpace(int Priority)
            return;
         // Remove the oldest file that has been "deleted":
         isyslog("low disk space while recording, trying to remove a deleted recording...");
-        cRecordings DeletedRecordings(true);
-        if (DeletedRecordings.Load()) {
+        cThreadLock DeletedRecordingsLock(&DeletedRecordings);
+        if (DeletedRecordings.Count()) {
            cRecording *r = DeletedRecordings.First();
            cRecording *r0 = r;
            while (r) {
@@ -119,13 +123,20 @@ void AssertFreeDiskSpace(int Priority)
                  r = DeletedRecordings.Next(r);
                  }
            if (r0 && r0->Remove()) {
+              DeletedRecordings.Del(r0);
               LastFreeDiskCheck += REMOVELATENCY / Factor;
               return;
               }
            }
+        // DeletedRecordings was empty, so to be absolutely sure there are no
+        // deleted recordings we need to double check:
+        DeletedRecordings.Update(true);
+        if (DeletedRecordings.Count())
+           return; // the next call will actually remove it
         // No "deleted" files to remove, so let's see if we can delete a recording:
         isyslog("...no deleted recording found, trying to delete an old recording...");
-        if (Recordings.Load()) {
+        cThreadLock RecordingsLock(&Recordings);
+        if (Recordings.Count()) {
            cRecording *r = Recordings.First();
            cRecording *r0 = NULL;
            while (r) {
@@ -199,11 +210,12 @@ int cResumeFile::Read(void)
 bool cResumeFile::Save(int Index)
 {
   if (fileName) {
-     int f = open(fileName, O_WRONLY | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
+     int f = open(fileName, O_WRONLY | O_CREAT | O_TRUNC, DEFFILEMODE);
      if (f >= 0) {
         if (safe_write(f, &Index, sizeof(Index)) < 0)
            LOG_ERROR_STR(fileName);
         close(f);
+        Recordings.ResetResume(fileName);
         return true;
         }
      }
@@ -215,16 +227,17 @@ void cResumeFile::Delete(void)
   if (fileName) {
      if (remove(fileName) < 0 && errno != ENOENT)
         LOG_ERROR_STR(fileName);
+     Recordings.ResetResume(fileName);
      }
 }
 
 // --- cRecordingInfo --------------------------------------------------------
 
-cRecordingInfo::cRecordingInfo(const cEvent *Event)
+cRecordingInfo::cRecordingInfo(tChannelID ChannelID, const cEvent *Event)
 {
+  channelID = ChannelID;
   if (Event) {
      event = Event;
-     channelID = event->ChannelID();
      ownEvent = NULL;
      }
   else
@@ -251,7 +264,9 @@ bool cRecordingInfo::Read(FILE *f)
   if (ownEvent) {
      cReadLine ReadLine;
      char *s;
+     int line = 0;
      while ((s = ReadLine.Read(f)) != NULL) {
+           ++line;
            char *t = skipspace(s + 1);
            switch (*s) {
              case 'C': {
@@ -262,8 +277,10 @@ bool cRecordingInfo::Read(FILE *f)
                             channelID = tChannelID::FromString(t);
                        }
                        break;
-             default: if (!ownEvent->Parse(s))
+             default: if (!ownEvent->Parse(s)) {
+                         esyslog("ERROR: EPG data problem in line %d", line);
                          return false;
+                         }
                       break;
              }
            }
@@ -293,83 +310,84 @@ tCharExchange CharExchange[] = {
   { 0, 0 }
   };
 
-static char *ExchangeChars(char *s, bool ToFileSystem)
+char *ExchangeChars(char *s, bool ToFileSystem)
 {
   char *p = s;
   while (*p) {
-#ifdef VFAT
-        // The VFAT file system can't handle all characters, so we
-        // have to take extra efforts to encode/decode them:
-        if (ToFileSystem) {
-           switch (*p) {
-                  // characters that can be used "as is":
-                  case '!':
-                  case '@':
-                  case '$':
-                  case '%':
-                  case '&':
-                  case '(':
-                  case ')':
-                  case '+':
-                  case ',':
-                  case '-':
-                  case ';':
-                  case '=':
-                  case '0' ... '9':
-                  case 'a' ... 'z':
-                  case 'A' ... 'Z':
-                  case '': case '':
-                  case '': case '':
-                  case '': case '':
-                  case '':
-                       break;
-                  // characters that can be mapped to other characters:
-                  case ' ': *p = '_'; break;
-                  case '~': *p = '/'; break;
-                  // characters that have to be encoded:
-                  default:
-                    if (*p != '.' || !*(p + 1) || *(p + 1) == '~') { // Windows can't handle '.' at the end of directory names
-                       int l = p - s;
-                       s = (char *)realloc(s, strlen(s) + 10);
-                       p = s + l;
-                       char buf[4];
-                       sprintf(buf, "#%02X", (unsigned char)*p);
-                       memmove(p + 2, p, strlen(p) + 1);
-                       strncpy(p, buf, 3);
-                       p += 2;
-                       }
-                  }
+        if (VfatFileSystem) {
+           // The VFAT file system can't handle all characters, so we
+           // have to take extra efforts to encode/decode them:
+           if (ToFileSystem) {
+              switch (*p) {
+                     // characters that can be used "as is":
+                     case '!':
+                     case '@':
+                     case '$':
+                     case '%':
+                     case '&':
+                     case '(':
+                     case ')':
+                     case '+':
+                     case ',':
+                     case '-':
+                     case ';':
+                     case '=':
+                     case '0' ... '9':
+                     case 'a' ... 'z':
+                     case 'A' ... 'Z':
+                     case '': case '':
+                     case '': case '':
+                     case '': case '':
+                     case '':
+                          break;
+                     // characters that can be mapped to other characters:
+                     case ' ': *p = '_'; break;
+                     case '~': *p = '/'; break;
+                     // characters that have to be encoded:
+                     default:
+                       if (*p != '.' || !*(p + 1) || *(p + 1) == '~') { // Windows can't handle '.' at the end of directory names
+                          int l = p - s;
+                          s = (char *)realloc(s, strlen(s) + 10);
+                          p = s + l;
+                          char buf[4];
+                          sprintf(buf, "#%02X", (unsigned char)*p);
+                          memmove(p + 2, p, strlen(p) + 1);
+                          strncpy(p, buf, 3);
+                          p += 2;
+                          }
+                     }
+              }
+           else {
+              switch (*p) {
+                // mapped characters:
+                case '_': *p = ' '; break;
+                case '/': *p = '~'; break;
+                // encodes characters:
+                case '#': {
+                     if (strlen(p) > 2) {
+                        char buf[3];
+                        sprintf(buf, "%c%c", *(p + 1), *(p + 2));
+                        unsigned char c = strtol(buf, NULL, 16);
+                        *p = c;
+                        memmove(p + 1, p + 3, strlen(p) - 2);
+                        }
+                     }
+                     break;
+                // backwards compatibility:
+                case '\x01': *p = '\''; break;
+                case '\x02': *p = '/';  break;
+                case '\x03': *p = ':';  break;
+                }
+              }
            }
         else {
-           switch (*p) {
-             // mapped characters:
-             case '_': *p = ' '; break;
-             case '/': *p = '~'; break;
-             // encodes characters:
-             case '#': {
-                  if (strlen(p) > 2) {
-                     char buf[3];
-                     sprintf(buf, "%c%c", *(p + 1), *(p + 2));
-                     unsigned char c = strtol(buf, NULL, 16);
-                     *p = c;
-                     memmove(p + 1, p + 3, strlen(p) - 2);
-                     }
-                  }
+           for (struct tCharExchange *ce = CharExchange; ce->a && ce->b; ce++) {
+               if (*p == (ToFileSystem ? ce->a : ce->b)) {
+                  *p = ToFileSystem ? ce->b : ce->a;
                   break;
-             // backwards compatibility:
-             case '\x01': *p = '\''; break;
-             case '\x02': *p = '/';  break;
-             case '\x03': *p = ':';  break;
-             }
-           }
-#else
-        for (struct tCharExchange *ce = CharExchange; ce->a && ce->b; ce++) {
-            if (*p == (ToFileSystem ? ce->a : ce->b)) {
-               *p = ToFileSystem ? ce->b : ce->a;
-               break;
+                  }
                }
-            }
-#endif
+           }
         p++;
         }
   return s;
@@ -424,7 +442,7 @@ cRecording::cRecording(cTimer *Timer, const cEvent *Event)
   priority = Timer->Priority();
   lifetime = Timer->Lifetime();
   // handle info:
-  info = new cRecordingInfo(Event);
+  info = new cRecordingInfo(Timer->Channel()->GetChannelID(), Event);
   // this is a somewhat ugly hack to get the 'summary' information from the
   // timer into the recording info, but it saves us from having to actually
   // copy the entire event data:
@@ -458,12 +476,14 @@ cRecording::cRecording(const char *FileName)
         name[p - FileName] = 0;
         name = ExchangeChars(name, false);
         }
+     GetResume();
      // read an optional info file:
      char *InfoFileName = NULL;
      asprintf(&InfoFileName, "%s%s", fileName, INFOFILESUFFIX);
      FILE *f = fopen(InfoFileName, "r");
      if (f) {
-        info->Read(f);
+        if (!info->Read(f))
+           esyslog("ERROR: EPG data problem in file %s", InfoFileName);
         fclose(f);
         }
      else if (errno != ENOENT)
@@ -496,10 +516,24 @@ cRecording::cRecording(const char *FileName)
                     line++;
                  }
            fclose(f);
-           if (line == 1) {
+           if (!data[2]) {
               data[2] = data[1];
               data[1] = NULL;
               }
+           else if (data[1] && data[2]) {
+              // if line 1 is too long, it can't be the short text,
+              // so assume the short text is missing and concatenate
+              // line 1 and line 2 to be the long text:
+              int len = strlen(data[1]);
+              if (len > 80) { 
+                 data[1] = (char *)realloc(data[1], len + 1 + strlen(data[2]) + 1);
+                 strcat(data[1], "\n");
+                 strcat(data[1], data[2]);
+                 free(data[2]);
+                 data[2] = data[1];
+                 data[1] = NULL;
+                 }
+              }
            info->SetData(data[0], data[1], data[2]);
            for (int i = 0; i < 3; i ++)
                free(data[i]);
@@ -705,21 +739,57 @@ bool cRecording::Remove(void)
   return RemoveVideoFile(FileName());
 }
 
+void cRecording::ResetResume(void) const
+{
+  resume = RESUME_NOT_INITIALIZED;
+}
+
 // --- cRecordings -----------------------------------------------------------
 
 cRecordings Recordings;
 
+char *cRecordings::updateFileName = NULL;
+
 cRecordings::cRecordings(bool Deleted)
+:cThread("video directory scanner")
 {
   deleted = Deleted;
   lastUpdate = 0;
+  state = 0;
 }
 
-void cRecordings::ScanVideoDir(const char *DirName)
+cRecordings::~cRecordings()
+{
+  Cancel(3);
+}
+
+void cRecordings::Action(void)
+{
+  Refresh();
+}
+
+const char *cRecordings::UpdateFileName(void)
+{
+  if (!updateFileName)
+     updateFileName = strdup(AddDirectory(VideoDirectory, ".update"));
+  return updateFileName;
+}
+
+void cRecordings::Refresh(bool Foreground)
+{
+  lastUpdate = time(NULL); // doing this first to make sure we don't miss anything
+  Lock();
+  Clear();
+  ChangeState();
+  Unlock();
+  ScanVideoDir(VideoDirectory, Foreground);
+}
+
+void cRecordings::ScanVideoDir(const char *DirName, bool Foreground)
 {
   cReadDir d(DirName);
   struct dirent *e;
-  while ((e = d.Next()) != NULL) {
+  while ((Foreground || Running()) && (e = d.Next()) != NULL) {
         if (strcmp(e->d_name, ".") && strcmp(e->d_name, "..")) {
            char *buffer;
            asprintf(&buffer, "%s/%s", DirName, e->d_name);
@@ -739,13 +809,17 @@ void cRecordings::ScanVideoDir(const char *DirName)
               if (S_ISDIR(st.st_mode)) {
                  if (endswith(buffer, deleted ? DELEXT : RECEXT)) {
                     cRecording *r = new cRecording(buffer);
-                    if (r->Name())
+                    if (r->Name()) {
+                       Lock();
                        Add(r);
+                       ChangeState();
+                       Unlock();
+                       }
                     else
                        delete r;
                     }
                  else
-                    ScanVideoDir(buffer);
+                    ScanVideoDir(buffer, Foreground);
                  }
               }
            free(buffer);
@@ -753,18 +827,34 @@ void cRecordings::ScanVideoDir(const char *DirName)
         }
 }
 
-bool cRecordings::NeedsUpdate(void)
+bool cRecordings::StateChanged(int &State)
 {
-  return lastUpdate <= LastModifiedTime(AddDirectory(VideoDirectory, ".update"));
+  int NewState = state;
+  bool Result = State != NewState;
+  State = state;
+  return Result;
 }
 
-bool cRecordings::Load(void)
+void cRecordings::TouchUpdate(void)
 {
-  lastUpdate = time(NULL); // doing this first to make sure we don't miss anything
-  Clear();
-  ScanVideoDir(VideoDirectory);
-  Sort();
-  return Count() > 0;
+  TouchFile(UpdateFileName());
+  lastUpdate = time(NULL); // make sure we don't tigger ourselves
+}
+
+bool cRecordings::NeedsUpdate(void)
+{
+  return lastUpdate < LastModifiedTime(UpdateFileName());
+}
+
+bool cRecordings::Update(bool Wait)
+{
+  if (Wait) {
+     Refresh(true);
+     return Count() > 0;
+     }
+  else
+     Start();
+  return false;
 }
 
 cRecording *cRecordings::GetByName(const char *FileName)
@@ -778,18 +868,35 @@ cRecording *cRecordings::GetByName(const char *FileName)
 
 void cRecordings::AddByName(const char *FileName)
 {
+  LOCK_THREAD;
   cRecording *recording = GetByName(FileName);
   if (!recording) {
      recording = new cRecording(FileName);
      Add(recording);
+     ChangeState();
+     TouchUpdate();
      }
 }
 
 void cRecordings::DelByName(const char *FileName)
 {
+  LOCK_THREAD;
   cRecording *recording = GetByName(FileName);
-  if (recording)
+  if (recording) {
      Del(recording);
+     ChangeState();
+     TouchUpdate();
+     }
+}
+
+void cRecordings::ResetResume(const char *ResumeFileName)
+{
+  LOCK_THREAD;
+  for (cRecording *recording = First(); recording; recording = Next(recording)) {
+      if (!ResumeFileName || strncmp(ResumeFileName, recording->FileName(), strlen(recording->FileName())) == 0)
+         recording->ResetResume();
+      }
+  ChangeState();
 }
 
 // --- cMark -----------------------------------------------------------------
@@ -974,7 +1081,7 @@ cIndexFile::cIndexFile(const char *FileName, bool Record)
         else if (!Record)
            isyslog("missing index file %s", fileName);
         if (Record) {
-           if ((f = open(fileName, O_WRONLY | O_CREAT | O_APPEND, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)) >= 0) {
+           if ((f = open(fileName, O_WRONLY | O_CREAT | O_APPEND, DEFFILEMODE)) >= 0) {
               if (delta) {
                  esyslog("ERROR: padding index file with %d '0' bytes", delta);
                  while (delta--)
@@ -1151,7 +1258,7 @@ int cIndexFile::Get(uchar FileNumber, int FileOffset)
 
 cFileName::cFileName(const char *FileName, bool Record, bool Blocking)
 {
-  file = -1;
+  file = NULL;
   fileNumber = 0;
   record = Record;
   blocking = Blocking;
@@ -1172,21 +1279,21 @@ cFileName::~cFileName()
   free(fileName);
 }
 
-int cFileName::Open(void)
+cUnbufferedFile *cFileName::Open(void)
 {
-  if (file < 0) {
+  if (!file) {
      int BlockingFlag = blocking ? 0 : O_NONBLOCK;
      if (record) {
         dsyslog("recording to '%s'", fileName);
         file = OpenVideoFile(fileName, O_RDWR | O_CREAT | BlockingFlag);
-        if (file < 0)
+        if (!file)
            LOG_ERROR_STR(fileName);
         }
      else {
         if (access(fileName, R_OK) == 0) {
            dsyslog("playing '%s'", fileName);
-           file = open(fileName, O_RDONLY | BlockingFlag);
-           if (file < 0)
+           file = cUnbufferedFile::Create(fileName, O_RDONLY | BlockingFlag);
+           if (!file)
               LOG_ERROR_STR(fileName);
            }
         else if (errno != ENOENT)
@@ -1198,14 +1305,14 @@ int cFileName::Open(void)
 
 void cFileName::Close(void)
 {
-  if (file >= 0) {
-     if ((record && CloseVideoFile(file) < 0) || (!record && close(file) < 0))
+  if (file) {
+     if ((record && CloseVideoFile(file) < 0) || (!record && file->Close() < 0))
         LOG_ERROR_STR(fileName);
-     file = -1;
+     file = NULL;
      }
 }
 
-int cFileName::SetOffset(int Number, int Offset)
+cUnbufferedFile *cFileName::SetOffset(int Number, int Offset)
 {
   if (fileNumber != Number)
      Close();
@@ -1221,7 +1328,7 @@ int cFileName::SetOffset(int Number, int Offset)
                  return SetOffset(Number + 1); // file exists and has non zero size, let's try next suffix
               else {
                  // zero size file, remove it
-                 dsyslog ("cFileName::SetOffset: removing zero-sized file %s\n", fileName);
+                 dsyslog ("cFileName::SetOffset: removing zero-sized file %s", fileName);
                  unlink (fileName);
                  }
               }
@@ -1230,23 +1337,23 @@ int cFileName::SetOffset(int Number, int Offset)
            }
         else if (errno != ENOENT) { // something serious has happened
            LOG_ERROR_STR(fileName);
-           return -1;
+           return NULL;
            }
         // found a non existing file suffix
         }
      if (Open() >= 0) {
-        if (!record && Offset >= 0 && lseek(file, Offset, SEEK_SET) != Offset) {
+        if (!record && Offset >= 0 && file && file->Seek(Offset, SEEK_SET) != Offset) {
            LOG_ERROR_STR(fileName);
-           return -1;
+           return NULL;
            }
         }
      return file;
      }
   esyslog("ERROR: max number of files (%d) exceeded", MAXFILESPERRECORDING);
-  return -1;
+  return NULL;
 }
 
-int cFileName::NextFile(void)
+cUnbufferedFile *cFileName::NextFile(void)
 {
   return SetOffset(fileNumber + 1);
 }
@@ -1280,7 +1387,7 @@ int SecondsToFrames(int Seconds)
 
 // --- ReadFrame -------------------------------------------------------------
 
-int ReadFrame(int f, uchar *b, int Length, int Max)
+int ReadFrame(cUnbufferedFile *f, uchar *b, int Length, int Max)
 {
   if (Length == -1)
      Length = Max; // this means we read up to EOF (see cIndex)
@@ -1288,10 +1395,8 @@ int ReadFrame(int f, uchar *b, int Length, int Max)
      esyslog("ERROR: frame larger than buffer (%d > %d)", Length, Max);
      Length = Max;
      }
-  int r = safe_read(f, b, Length);
+  int r = f->Read(b, Length);
   if (r < 0)
      LOG_ERROR;
   return r;
 }
-
-
