@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: recording.h 1.38 2005/05/28 09:34:07 kls Exp $
+ * $Id: recording.h 1.46 2005/10/31 12:27:12 kls Exp $
  */
 
 #ifndef __RECORDING_H
@@ -18,6 +18,8 @@
 #include "timers.h"
 #include "tools.h"
 
+extern bool VfatFileSystem;
+
 void RemoveDeletedRecordings(void);
 void AssertFreeDiskSpace(int Priority = 0);
      ///< The special Priority value -1 means that we shall get rid of any
@@ -40,7 +42,7 @@ private:
   tChannelID channelID;
   const cEvent *event;
   cEvent *ownEvent;
-  cRecordingInfo(const cEvent *Event = NULL);
+  cRecordingInfo(tChannelID ChannelID = tChannelID::InvalidID, const cEvent *Event = NULL);
   void SetData(const char *Title, const char *ShortText, const char *Description);
 public:
   ~cRecordingInfo();
@@ -69,7 +71,7 @@ public:
   int lifetime;
   cRecording(cTimer *Timer, const cEvent *Event);
   cRecording(const char *FileName);
-  ~cRecording();
+  virtual ~cRecording();
   virtual int Compare(const cListObject &ListObject) const;
   const char *Name(void) const { return name; }
   const char *FileName(void) const;
@@ -77,6 +79,7 @@ public:
   const cRecordingInfo *Info(void) const { return info; }
   const char *PrefixFileName(char Prefix);
   int HierarchyLevels(void) const;
+  void ResetResume(void) const;
   bool IsNew(void) const { return GetResume() <= 0; }
   bool IsEdited(void) const;
   bool WriteInfo(void);
@@ -88,16 +91,38 @@ public:
        // Returns false in case of error
   };
 
-class cRecordings : public cList<cRecording> {
+class cRecordings : public cList<cRecording>, public cThread {
 private:
+  static char *updateFileName;
   bool deleted;
   time_t lastUpdate;
-  void ScanVideoDir(const char *DirName);
+  int state;
+  const char *UpdateFileName(void);
+  void Refresh(bool Foreground = false);
+  void ScanVideoDir(const char *DirName, bool Foreground = false);
+protected:
+  void Action(void);
 public:
   cRecordings(bool Deleted = false);
-  bool Load(void);
-  void TriggerUpdate(void) { lastUpdate = 0; }
+  virtual ~cRecordings();
+  bool Load(void) { return Update(true); }
+       ///< Loads the current list of recordings and returns true if there
+       ///< is anything in it (for compatibility with older plugins - use
+       ///< Update(true) instead).
+  bool Update(bool Wait = false);
+       ///< Triggers an update of the list of recordings, which will run
+       ///< as a separate thread if Wait is false. If Wait is true, the
+       ///< function returns only after the update has completed.
+       ///< Returns true if Wait is true and there is anyting in the list
+       ///< of recordings, false otherwise.
+  void TouchUpdate(void);
+       ///< Touches the '.update' file in the video directory, so that other
+       ///< instances of VDR that access the same video directory can be triggered
+       ///< to update their recordings list.
   bool NeedsUpdate(void);
+  void ChangeState(void) { state++; }
+  bool StateChanged(int &State);
+  void ResetResume(const char *ResumeFileName = NULL);
   cRecording *GetByName(const char *FileName);
   void AddByName(const char *FileName);
   void DelByName(const char *FileName);
@@ -110,7 +135,7 @@ public:
   int position;
   char *comment;
   cMark(int Position = 0, const char *Comment = NULL);
-  ~cMark();
+  virtual ~cMark();
   cString ToText(void);
   bool Parse(const char *s);
   bool Save(FILE *f);
@@ -178,7 +203,7 @@ public:
 
 class cFileName {
 private:
-  int file;
+  cUnbufferedFile *file;
   int fileNumber;
   char *fileName, *pFileNumber;
   bool record;
@@ -188,10 +213,10 @@ public:
   ~cFileName();
   const char *Name(void) { return fileName; }
   int Number(void) { return fileNumber; }
-  int Open(void);
+  cUnbufferedFile *Open(void);
   void Close(void);
-  int SetOffset(int Number, int Offset = 0);
-  int NextFile(void);
+  cUnbufferedFile *SetOffset(int Number, int Offset = 0);
+  cUnbufferedFile *NextFile(void);
   };
 
 cString IndexToHMSF(int Index, bool WithFrame = false);
@@ -201,6 +226,12 @@ int HMSFToIndex(const char *HMSF);
 int SecondsToFrames(int Seconds); //XXX+ ->player???
       // Returns the number of frames corresponding to the given number of seconds.
 
-int ReadFrame(int f, uchar *b, int Length, int Max);
+int ReadFrame(cUnbufferedFile *f, uchar *b, int Length, int Max);
+
+char *ExchangeChars(char *s, bool ToFileSystem);
+      // Exchanges the characters in the given string to or from a file system
+      // specific representation (depending on ToFileSystem). The given string will
+      // be modified and may be reallocated if more space is needed. The return
+      // value points to the resulting string, which may be different from s.
 
 #endif //__RECORDING_H
