@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: remote.c 1.42 2005/03/20 13:25:31 kls Exp $
+ * $Id: remote.c 1.45 2005/09/03 12:29:48 kls Exp $
  */
 
 #include "remote.h"
@@ -143,6 +143,12 @@ bool cRemote::Put(const char *Code, bool Repeat, bool Release)
   return false;
 }
 
+void cRemote::CallPlugin(const char *Plugin)
+{
+  plugin = Plugin;
+  Put(k_Plugin);
+}
+
 bool cRemote::HasKeys(void)
 {
   cMutexLock MutexLock(&mutex);
@@ -213,7 +219,6 @@ cKbdRemote::cKbdRemote(void)
 :cRemote("KBD")
 ,cThread("KBD remote control")
 {
-  active = false;
   tcgetattr(STDIN_FILENO, &savedTm);
   struct termios tm;
   if (tcgetattr(STDIN_FILENO, &tm) == 0) {
@@ -230,7 +235,6 @@ cKbdRemote::cKbdRemote(void)
 cKbdRemote::~cKbdRemote()
 {
   kbdAvailable = false;
-  active = false;
   Cancel(3);
   tcsetattr(STDIN_FILENO, TCSANOW, &savedTm);
 }
@@ -261,12 +265,11 @@ int cKbdRemote::MapCodeToFunc(uint64 Code)
 void cKbdRemote::Action(void)
 {
   cPoller Poller(STDIN_FILENO);
-  active = true;
-  while (active) {
+  while (Running()) {
         if (Poller.Poll(100)) {
            uint64 Command = 0;
            uint i = 0;
-           while (active && i < sizeof(Command)) {
+           while (Running() && i < sizeof(Command)) {
                  uchar ch;
                  int r = read(STDIN_FILENO, &ch, 1);
                  if (r == 1) {
