@@ -10,7 +10,7 @@
 # See the main source file 'vdr.c' for copyright information and
 # how to reach the author.
 #
-# $Id: summary2info.pl 1.3 2005/06/04 11:33:09 kls Exp $
+# $Id: summary2info.pl 1.5 2005/09/17 09:20:31 kls Exp $
 
 $VideoDir = $ARGV[0] || die "please provide the name of the video directory\n";
 
@@ -33,10 +33,19 @@ for $SummaryFile (@SummaryFiles) {
              }
           }
     close(F);
-    if ($line == 1) {
+    if (!$data[2]) {
        $data[2] = $data[1];
        $data[1] = "";
        }
+    elsif ($data[1] && $data[2]) {
+       # if line 1 is too long, it can't be the short text,
+       # so assume the short text is missing and concatenate
+       # line 1 and line 2 to be the long text:
+       if (length($data[1]) > 80) { 
+          $data[2] = $data[1] . "|" . $data[2];
+          $data[1] = "";
+          }
+       }
     ($InfoFile = $SummaryFile) =~ s/summary\.vdr$/info.vdr/;
     open(F, ">$InfoFile") || die "$InfoFile: $!\n";
     print F "T $data[0]\n" if ($data[0]);
