@@ -10,7 +10,7 @@
  * and interact with the Video Disk Recorder - or write a full featured
  * graphical interface that sits on top of an SVDRP connection.
  *
- * $Id: svdrp.c 1.72 2005/05/26 09:59:09 kls Exp $
+ * $Id: svdrp.c 1.84 2005/11/27 15:29:28 kls Exp $
  */
 
 #include "svdrp.h"
@@ -28,11 +28,14 @@
 #include <unistd.h>
 #include "channels.h"
 #include "config.h"
+#include "cutter.h"
 #include "device.h"
 #include "eitscan.h"
 #include "keys.h"
 #include "menu.h"
+#include "plugin.h"
 #include "remote.h"
+#include "skins.h"
 #include "timers.h"
 #include "tools.h"
 #include "videodir.h"
@@ -194,6 +197,10 @@ const char *HelpPages[] = {
   "    RECORDING - BE SURE YOU KNOW WHAT YOU ARE DOING!",
   "DELT <number>\n"
   "    Delete timer.",
+  "EDIT <number>\n"
+  "    Edit the recording with the given number. Before a recording can be\n"
+  "    edited, an LSTR command must have been executed in order to retrieve\n"
+  "    the recording numbers.",
   "GRAB <filename> [ jpeg | pnm [ <quality> [ <sizex> <sizey> ] ] ]\n"
   "    Grab the current frame and save it to the given file. Images can\n"
   "    be stored as JPEG (default) or PNM, at the given quality (default\n"
@@ -209,7 +216,7 @@ const char *HelpPages[] = {
   "    containing the given string as part of their name are listed.",
   "LSTE [ <channel> ] [ now | next | at <time> ]\n"
   "    List EPG data. Without any parameters all data of all channels is\n"
-  "    listed. If a channel is given (either by number of by channel ID),\n"
+  "    listed. If a channel is given (either by number or by channel ID),\n"
   "    only data for that channel is listed. 'now', 'next', or 'at <time>'\n"
   "    restricts the returned data to present events, following events, or\n"
   "    events at the given time (which must be in time_t form).",
@@ -219,12 +226,9 @@ const char *HelpPages[] = {
   "LSTT [ <number> ]\n"
   "    List timers. Without option, all timers are listed. Otherwise\n"
   "    only the given timer is listed.",
-  "MESG [ <message> ]\n"
-  "    Displays the given message on the OSD. If message is omitted, the\n"
-  "    currently pending message (if any) will be returned. The message\n"
-  "    will be displayed for a few seconds as soon as the OSD has become\n"
-  "    idle. If a new MESG command is entered while the previous message\n"
-  "    has not yet been displayed, the old message will be overwritten.",
+  "MESG <message>\n"
+  "    Displays the given message on the OSD. The message will be queued\n"
+  "    and displayed whenever this is suitable.\n",
   "MODC <number> <settings>\n"
   "    Modify a channel. Settings must be in the same format as returned\n"
   "    by the LSTC command.",
@@ -253,6 +257,26 @@ const char *HelpPages[] = {
   "    zero, this means that the timer is currently recording and has started\n"
   "    at the given time. The first value in the resulting line is the number\n"
   "    of the timer.",
+  "PLAY <number> [ begin | <position> ]\n"
+  "    Play the recording with the given number. Before a recording can be\n"
+  "    played, an LSTR command must have been executed in order to retrieve\n"
+  "    the recording numbers.\n"
+  "    The keyword 'begin' plays the recording from its very beginning, while\n"
+  "    a <position> (given as hh:mm:ss[.ff] or framenumber) starts at that\n"
+  "    position. If neither 'begin' nor a <position> are given, replay is resumed\n"
+  "    at the position where any previous replay was stopped, or from the beginning\n"
+  "    by default. To control or stop the replay session, use the usual remote\n"
+  "    control keypresses via the HITK command.",
+  "PLUG <name> [ help | main ] [ <command> [ <options> ]]\n"
+  "    Send a command to a plugin.\n"
+  "    The PLUG command without any parameters lists all plugins.\n"
+  "    If only a name is given, all commands known to that plugin are listed.\n"
+  "    If a command is given (optionally followed by parameters), that command\n"
+  "    is sent to the plugin, and the result will be displayed.\n"
+  "    The keyword 'help' lists all the SVDRP commands known to the named plugin.\n"
+  "    If 'help' is followed by a command, the detailed help for that command is\n"
+  "    given. The keyword 'main' initiates a call to the main menu function of the\n"
+  "    given plugin.\n",
   "PUTE\n"
   "    Put data into the EPG list. The data entered has to strictly follow the\n"
   "    format defined in vdr(5) for the 'epg.data' file.  A '.' on a line\n"
@@ -294,6 +318,8 @@ const char *HelpPages[] = {
  504 Command parameter not implemented
  550 Requested action not taken
  554 Transaction failed
+ 900 Default plugin reply code
+ 901..999 Plugin specific reply codes
 
 */
 
@@ -315,15 +341,16 @@ const char *GetHelpTopic(const char *HelpPage)
   return NULL;
 }
 
-const char *GetHelpPage(const char *Cmd)
+const char *GetHelpPage(const char *Cmd, const char **p)
 {
-  const char **p = HelpPages;
-  while (*p) {
-        const char *t = GetHelpTopic(*p);
-        if (strcasecmp(Cmd, t) == 0)
-           return *p;
-        p++;
-        }
+  if (p) {
+     while (*p) {
+           const char *t = GetHelpTopic(*p);
+           if (strcasecmp(Cmd, t) == 0)
+              return *p;
+           p++;
+           }
+     }
   return NULL;
 }
 
@@ -332,7 +359,8 @@ cSVDRP::cSVDRP(int Port)
 {
   PUTEhandler = NULL;
   numChars = 0;
-  message = NULL;
+  length = BUFSIZ;
+  cmdLine = MALLOC(char, length);
   lastActivity = 0;
   isyslog("SVDRP listening on port %d", Port);
 }
@@ -340,7 +368,7 @@ cSVDRP::cSVDRP(int Port)
 cSVDRP::~cSVDRP()
 {
   Close();
-  free(message);
+  free(cmdLine);
 }
 
 void cSVDRP::Close(bool Timeout)
@@ -376,14 +404,14 @@ void cSVDRP::Reply(int Code, const char *fmt, ...)
         va_start(ap, fmt);
         char *buffer;
         vasprintf(&buffer, fmt, ap);
-        char *nl = strchr(buffer, '\n');
-        if (Code > 0 && nl && *(nl + 1)) // trailing newlines don't count!
-           Code = -Code;
-        char number[16];
-        sprintf(number, "%03d%c", abs(Code), Code < 0 ? '-' : ' ');
         const char *s = buffer;
         while (s && *s) {
               const char *n = strchr(s, '\n');
+              char cont = ' ';
+              if (Code < 0 || n && *(n + 1)) // trailing newlines don't count!
+                 cont = '-';
+              char number[16];
+              sprintf(number, "%03d%c", abs(Code), cont);
               if (!(Send(number) && Send(s, n ? n - s : -1) && Send("\r\n"))) {
                  Close();
                  break;
@@ -400,6 +428,32 @@ void cSVDRP::Reply(int Code, const char *fmt, ...)
      }
 }
 
+void cSVDRP::PrintHelpTopics(const char **hp)
+{
+  int NumPages = 0;
+  if (hp) {
+     while (*hp) {
+           NumPages++;
+           hp++;
+           }
+     hp -= NumPages;
+     }
+  const int TopicsPerLine = 5;
+  int x = 0;
+  for (int y = 0; (y * TopicsPerLine + x) < NumPages; y++) {
+      char buffer[TopicsPerLine * MAXHELPTOPIC + 5];
+      char *q = buffer;
+      q += sprintf(q, "    ");
+      for (x = 0; x < TopicsPerLine && (y * TopicsPerLine + x) < NumPages; x++) {
+          const char *topic = GetHelpTopic(hp[(y * TopicsPerLine + x)]);
+          if (topic)
+             q += sprintf(q, "%*s", -MAXHELPTOPIC, topic);
+          }
+      x = 0;
+      Reply(-214, buffer);
+      }
+}
+
 void cSVDRP::CmdCHAN(const char *Option)
 {
   if (*Option) {
@@ -476,22 +530,26 @@ void cSVDRP::CmdDELC(const char *Option)
 {
   if (*Option) {
      if (isnumber(Option)) {
-        cChannel *channel = Channels.GetByNumber(strtol(Option, NULL, 10));
-        if (channel) {
-           for (cTimer *timer = Timers.First(); timer; timer = Timers.Next(timer)) {
-               if (timer->Channel() == channel) {
-                  Reply(550, "Channel \"%s\" is in use by timer %d", Option, timer->Index() + 1);
-                  return;
+        if (!Channels.BeingEdited()) {
+           cChannel *channel = Channels.GetByNumber(strtol(Option, NULL, 10));
+           if (channel) {
+              for (cTimer *timer = Timers.First(); timer; timer = Timers.Next(timer)) {
+                  if (timer->Channel() == channel) {
+                     Reply(550, "Channel \"%s\" is in use by timer %d", Option, timer->Index() + 1);
+                     return;
+                     }
                   }
-               }
-           Channels.Del(channel);
-           Channels.ReNumber();
-           Channels.SetModified(true);
-           isyslog("channel %s deleted", Option);
-           Reply(250, "Channel \"%s\" deleted", Option);
+              Channels.Del(channel);
+              Channels.ReNumber();
+              Channels.SetModified(true);
+              isyslog("channel %s deleted", Option);
+              Reply(250, "Channel \"%s\" deleted", Option);
+              }
+           else
+              Reply(501, "Channel \"%s\" not defined", Option);
            }
         else
-           Reply(501, "Channel \"%s\" not defined", Option);
+           Reply(550, "Channels are being edited - try again later");
         }
      else
         Reply(501, "Error in channel number \"%s\"", Option);
@@ -510,7 +568,7 @@ void cSVDRP::CmdDELR(const char *Option)
            if (!rc) {
               if (recording->Delete()) {
                  Reply(250, "Recording \"%s\" deleted", Option);
-                 ::Recordings.Load(); // must make sure the global recordings list is updated
+                 ::Recordings.DelByName(recording->FileName());
                  }
               else
                  Reply(554, "Error while deleting recording!");
@@ -532,19 +590,23 @@ void cSVDRP::CmdDELT(const char *Option)
 {
   if (*Option) {
      if (isnumber(Option)) {
-        cTimer *timer = Timers.Get(strtol(Option, NULL, 10) - 1);
-        if (timer) {
-           if (!timer->Recording()) {
-              isyslog("deleting timer %s", *timer->ToDescr());
-              Timers.Del(timer);
-              Timers.SetModified();
-              Reply(250, "Timer \"%s\" deleted", Option);
+        if (!Timers.BeingEdited()) {
+           cTimer *timer = Timers.Get(strtol(Option, NULL, 10) - 1);
+           if (timer) {
+              if (!timer->Recording()) {
+                 isyslog("deleting timer %s", *timer->ToDescr());
+                 Timers.Del(timer);
+                 Timers.SetModified();
+                 Reply(250, "Timer \"%s\" deleted", Option);
+                 }
+              else
+                 Reply(550, "Timer \"%s\" is recording", Option);
               }
            else
-              Reply(550, "Timer \"%s\" is recording", Option);
+              Reply(501, "Timer \"%s\" not defined", Option);
            }
         else
-           Reply(501, "Timer \"%s\" not defined", Option);
+           Reply(550, "Timers are being edited - try again later");
         }
      else
         Reply(501, "Error in timer number \"%s\"", Option);
@@ -553,6 +615,36 @@ void cSVDRP::CmdDELT(const char *Option)
      Reply(501, "Missing timer number");
 }
 
+void cSVDRP::CmdEDIT(const char *Option)
+{
+  if (*Option) {
+     if (isnumber(Option)) {
+        cRecording *recording = Recordings.Get(strtol(Option, NULL, 10) - 1);
+        if (recording) {
+           cMarks Marks;
+           if (Marks.Load(recording->FileName()) && Marks.Count()) {
+              if (!cCutter::Active()) {
+                 if (cCutter::Start(recording->FileName()))
+                    Reply(250, "Editing recording \"%s\" [%s]", Option, recording->Title());
+                 else
+                    Reply(554, "Can't start editing process");
+                 }
+              else
+                 Reply(554, "Editing process already active");
+              }
+           else
+              Reply(554, "No editing marks defined");
+           }
+        else
+           Reply(550, "Recording \"%s\" not found%s", Option, Recordings.Count() ? "" : " (use LSTR before editing)");
+        }
+     else
+        Reply(501, "Error in recording number \"%s\"", Option);
+     }
+  else
+     Reply(501, "Missing recording number");
+}
+
 void cSVDRP::CmdGRAB(const char *Option)
 {
   char *FileName = NULL;
@@ -618,7 +710,7 @@ void cSVDRP::CmdGRAB(const char *Option)
 void cSVDRP::CmdHELP(const char *Option)
 {
   if (*Option) {
-     const char *hp = GetHelpPage(Option);
+     const char *hp = GetHelpPage(Option, HelpPages);
      if (hp)
         Reply(214, hp);
      else {
@@ -629,24 +721,13 @@ void cSVDRP::CmdHELP(const char *Option)
   else {
      Reply(-214, "This is VDR version %s", VDRVERSION);
      Reply(-214, "Topics:");
-     const char **hp = HelpPages;
-     int NumPages = 0;
-     while (*hp) {
-           NumPages++;
-           hp++;
-           }
-     const int TopicsPerLine = 5;
-     int x = 0;
-     for (int y = 0; (y * TopicsPerLine + x) < NumPages; y++) {
-         char buffer[TopicsPerLine * (MAXHELPTOPIC + 5)];
-         char *q = buffer;
-         for (x = 0; x < TopicsPerLine && (y * TopicsPerLine + x) < NumPages; x++) {
-             const char *topic = GetHelpTopic(HelpPages[(y * TopicsPerLine + x)]);
-             if (topic)
-                q += sprintf(q, "    %s", topic);
-             }
-         x = 0;
-         Reply(-214, buffer);
+     PrintHelpTopics(HelpPages);
+     cPlugin *plugin;
+     for (int i = 0; (plugin = cPluginManager::GetPlugin(i)) != NULL; i++) {
+         const char **hp = plugin->SVDRPHelpPages();
+         if (hp)
+            Reply(-214, "Plugin %s v%s - %s", plugin->Name(), plugin->Version(), plugin->Description());
+         PrintHelpTopics(hp);
          }
      Reply(-214, "To report bugs in the implementation send email to");
      Reply(-214, "    vdr-bugs@cadsoft.de");
@@ -782,18 +863,25 @@ void cSVDRP::CmdLSTE(const char *Option)
               p = strtok_r(NULL, delim, &strtok_next);
               }
         }
-     FILE *f = fdopen(file, "w");
-     if (f) {
-        if (Schedule)
-           Schedule->Dump(f, "215-", DumpMode, AtTime);
-        else
-           Schedules->Dump(f, "215-", DumpMode, AtTime);
-        fflush(f);
-        Reply(215, "End of EPG data");
-        // don't 'fclose(f)' here!
+     int fd = dup(file);
+     if (fd) {
+        FILE *f = fdopen(fd, "w");
+        if (f) {
+           if (Schedule)
+              Schedule->Dump(f, "215-", DumpMode, AtTime);
+           else
+              Schedules->Dump(f, "215-", DumpMode, AtTime);
+           fflush(f);
+           Reply(215, "End of EPG data");
+           fclose(f);
+           }
+        else {
+           Reply(451, "Can't open file connection");
+           close(fd);
+           }
         }
      else
-        Reply(451, "Can't open file connection");
+        Reply(451, "Can't dup stream descriptor");
      }
   else
      Reply(451, "Can't get EPG data");
@@ -801,7 +889,7 @@ void cSVDRP::CmdLSTE(const char *Option)
 
 void cSVDRP::CmdLSTR(const char *Option)
 {
-  bool recordings = Recordings.Load();
+  bool recordings = Recordings.Update(true);
   if (*Option) {
      if (isnumber(Option)) {
         cRecording *recording = Recordings.Get(strtol(Option, NULL, 10) - 1);
@@ -862,15 +950,12 @@ void cSVDRP::CmdLSTT(const char *Option)
 void cSVDRP::CmdMESG(const char *Option)
 {
   if (*Option) {
-     free(message);
-     message = strdup(Option);
-     isyslog("SVDRP message: '%s'", message);
-     Reply(250, "Message stored");
+     isyslog("SVDRP message: '%s'", Option);
+     Skins.QueueMessage(mtInfo, Option);
+     Reply(250, "Message queued");
      }
-  else if (message)
-     Reply(250, "%s", message);
   else
-     Reply(550, "No pending message");
+     Reply(501, "Missing message");
 }
 
 void cSVDRP::CmdMODC(const char *Option)
@@ -880,25 +965,29 @@ void cSVDRP::CmdMODC(const char *Option)
      int n = strtol(Option, &tail, 10);
      if (tail && tail != Option) {
         tail = skipspace(tail);
-        cChannel *channel = Channels.GetByNumber(n);
-        if (channel) {
-           cChannel ch;
-           if (ch.Parse(tail)) {
-              if (Channels.HasUniqueChannelID(&ch, channel)) {
-                 *channel = ch;
-                 Channels.ReNumber();
-                 Channels.SetModified(true);
-                 isyslog("modifed channel %d %s", channel->Number(), *channel->ToText());
-                 Reply(250, "%d %s", channel->Number(), *channel->ToText());
+        if (!Channels.BeingEdited()) {
+           cChannel *channel = Channels.GetByNumber(n);
+           if (channel) {
+              cChannel ch;
+              if (ch.Parse(tail)) {
+                 if (Channels.HasUniqueChannelID(&ch, channel)) {
+                    *channel = ch;
+                    Channels.ReNumber();
+                    Channels.SetModified(true);
+                    isyslog("modifed channel %d %s", channel->Number(), *channel->ToText());
+                    Reply(250, "%d %s", channel->Number(), *channel->ToText());
+                    }
+                 else
+                    Reply(501, "Channel settings are not unique");
                  }
               else
-                 Reply(501, "Channel settings are not unique");
+                 Reply(501, "Error in channel settings");
               }
            else
-              Reply(501, "Error in channel settings");
+              Reply(501, "Channel \"%d\" not defined", n);
            }
         else
-           Reply(501, "Channel \"%d\" not defined", n);
+           Reply(550, "Channels are being edited - try again later");
         }
      else
         Reply(501, "Error in channel number");
@@ -914,24 +1003,28 @@ void cSVDRP::CmdMODT(const char *Option)
      int n = strtol(Option, &tail, 10);
      if (tail && tail != Option) {
         tail = skipspace(tail);
-        cTimer *timer = Timers.Get(n - 1);
-        if (timer) {
-           cTimer t = *timer;
-           if (strcasecmp(tail, "ON") == 0)
-              t.SetFlags(tfActive);
-           else if (strcasecmp(tail, "OFF") == 0)
-              t.ClrFlags(tfActive);
-           else if (!t.Parse(tail)) {
-              Reply(501, "Error in timer settings");
-              return;
+        if (!Timers.BeingEdited()) {
+           cTimer *timer = Timers.Get(n - 1);
+           if (timer) {
+              cTimer t = *timer;
+              if (strcasecmp(tail, "ON") == 0)
+                 t.SetFlags(tfActive);
+              else if (strcasecmp(tail, "OFF") == 0)
+                 t.ClrFlags(tfActive);
+              else if (!t.Parse(tail)) {
+                 Reply(501, "Error in timer settings");
+                 return;
+                 }
+              *timer = t;
+              Timers.SetModified();
+              isyslog("timer %s modified (%s)", *timer->ToDescr(), timer->HasFlags(tfActive) ? "active" : "inactive");
+              Reply(250, "%d %s", timer->Index() + 1, *timer->ToText());
               }
-           *timer = t;
-           Timers.SetModified();
-           isyslog("timer %s modified (%s)", *timer->ToDescr(), timer->HasFlags(tfActive) ? "active" : "inactive");
-           Reply(250, "%d %s", timer->Index() + 1, *timer->ToText());
+           else
+              Reply(501, "Timer \"%d\" not defined", n);
            }
         else
-           Reply(501, "Timer \"%d\" not defined", n);
+           Reply(550, "Timers are being edited - try again later");
         }
      else
         Reply(501, "Error in timer number");
@@ -942,8 +1035,51 @@ void cSVDRP::CmdMODT(const char *Option)
 
 void cSVDRP::CmdMOVC(const char *Option)
 {
-  //TODO combine this with menu action (timers must be updated)
-  Reply(502, "MOVC not yet implemented");
+  if (*Option) {
+     if (!Channels.BeingEdited() && !Timers.BeingEdited()) {
+        char *tail;
+        int From = strtol(Option, &tail, 10);
+        if (tail && tail != Option) {
+           tail = skipspace(tail);
+           if (tail && tail != Option) {
+              int To = strtol(tail, NULL, 10);
+              int CurrentChannelNr = cDevice::CurrentChannel();
+              cChannel *CurrentChannel = Channels.GetByNumber(CurrentChannelNr);
+              cChannel *FromChannel = Channels.GetByNumber(From);
+              if (FromChannel) {
+                 cChannel *ToChannel = Channels.GetByNumber(To);
+                 if (ToChannel) {
+                    int FromNumber = FromChannel->Number();
+                    int ToNumber = ToChannel->Number();
+                    if (FromNumber != ToNumber) {
+                       Channels.Move(FromChannel, ToChannel);
+                       Channels.ReNumber();
+                       Channels.SetModified(true);
+                       if (CurrentChannel && CurrentChannel->Number() != CurrentChannelNr)
+                          Channels.SwitchTo(CurrentChannel->Number());
+                       isyslog("channel %d moved to %d", FromNumber, ToNumber);
+                       Reply(250,"Channel \"%d\" moved to \"%d\"", From, To);
+                       }
+                    else
+                       Reply(501, "Can't move channel to same postion");
+                    }
+                 else
+                    Reply(501, "Channel \"%d\" not defined", To);
+                 }
+              else
+                 Reply(501, "Channel \"%d\" not defined", From);
+              }
+           else
+              Reply(501, "Error in channel number");
+           }
+        else
+           Reply(501, "Error in channel number");
+        }
+     else
+        Reply(550, "Channels or timers are being edited - try again later");
+     }
+  else
+     Reply(501, "Missing channel number");
 }
 
 void cSVDRP::CmdMOVT(const char *Option)
@@ -1019,6 +1155,124 @@ void cSVDRP::CmdNEXT(const char *Option)
      Reply(550, "No active timers");
 }
 
+void cSVDRP::CmdPLAY(const char *Option)
+{
+  if (*Option) {
+     char *opt = strdup(Option);
+     char *num = skipspace(opt);
+     char *option = num;
+     while (*option && !isspace(*option))
+           option++;
+     char c = *option;
+     *option = 0;
+     if (isnumber(num)) {
+        cRecording *recording = Recordings.Get(strtol(num, NULL, 10) - 1);
+        if (recording) {
+           if (c)
+              option = skipspace(++option);
+           cReplayControl::SetRecording(NULL, NULL);
+           cControl::Shutdown();
+           if (*option) {
+              int pos = 0;
+              if (strcasecmp(option, "BEGIN") != 0) {
+                 int h, m = 0, s = 0, f = 1;
+                 int x = sscanf(option, "%d:%d:%d.%d", &h, &m, &s, &f);
+                 if (x == 1)
+                    pos = h;
+                 else if (x >= 3)
+                    pos = (h * 3600 + m * 60 + s) * FRAMESPERSEC + f - 1;
+                 }
+              cResumeFile resume(recording->FileName());
+              if (pos <= 0)
+                 resume.Delete();
+              else
+                 resume.Save(pos);
+              }
+           cReplayControl::SetRecording(recording->FileName(), recording->Title());
+           cControl::Launch(new cReplayControl);
+           cControl::Attach();
+           Reply(250, "Playing recording \"%s\" [%s]", num, recording->Title());
+           }
+        else
+           Reply(550, "Recording \"%s\" not found%s", num, Recordings.Count() ? "" : " (use LSTR before playing)");
+        }
+     else
+        Reply(501, "Error in recording number \"%s\"", num);
+     free(opt);
+     }
+  else
+     Reply(501, "Missing recording number");
+}
+
+void cSVDRP::CmdPLUG(const char *Option)
+{
+  if (*Option) {
+     char *opt = strdup(Option);
+     char *name = skipspace(opt);
+     char *option = name;
+     while (*option && !isspace(*option))
+        option++;
+     char c = *option;
+     *option = 0;
+     cPlugin *plugin = cPluginManager::GetPlugin(name);
+     if (plugin) {
+        if (c)
+           option = skipspace(++option);
+        char *cmd = option;
+        while (*option && !isspace(*option))
+              option++;
+        if (*option) {
+           *option++ = 0;
+           option = skipspace(option);
+           }
+        if (!*cmd || strcasecmp(cmd, "HELP") == 0) {
+           if (*cmd && *option) {
+              const char *hp = GetHelpPage(option, plugin->SVDRPHelpPages());
+              if (hp) {
+                 Reply(-214, hp);
+                 Reply(214, "End of HELP info");
+                 }
+              else
+                 Reply(504, "HELP topic \"%s\" for plugin \"%s\" unknown", option, plugin->Name());
+              }
+           else {
+              Reply(-214, "Plugin %s v%s - %s", plugin->Name(), plugin->Version(), plugin->Description());
+              const char **hp = plugin->SVDRPHelpPages();
+              if (hp) {
+                 Reply(-214, "SVDRP commands:");
+                 PrintHelpTopics(hp);
+                 Reply(214, "End of HELP info");
+                 }
+              else
+                 Reply(214, "This plugin has no SVDRP commands");
+              }
+           }
+        else if (strcasecmp(cmd, "MAIN") == 0) {
+           cRemote::CallPlugin(plugin->Name());
+           Reply(250, "Initiated call to main menu function of plugin \"%s\"", plugin->Name());
+           }
+        else {
+           int ReplyCode = 900;
+           cString s = plugin->SVDRPCommand(cmd, option, ReplyCode);
+           if (s)
+              Reply(abs(ReplyCode), *s);
+           else
+              Reply(500, "Command unrecognized: \"%s\"", cmd);
+           }
+        }
+     else
+        Reply(550, "Plugin \"%s\" not found (use PLUG for a list of plugins)", name);
+     free(opt);
+     }
+  else {
+     Reply(-214, "Available plugins:");
+     cPlugin *plugin;
+     for (int i = 0; (plugin = cPluginManager::GetPlugin(i)) != NULL; i++)
+         Reply(-214, "%s v%s - %s", plugin->Name(), plugin->Version(), plugin->Description());
+     Reply(214, "End of plugin list");
+     }
+}
+
 void cSVDRP::CmdPUTE(const char *Option)
 {
   delete PUTEhandler;
@@ -1054,20 +1308,24 @@ void cSVDRP::CmdUPDT(const char *Option)
   if (*Option) {
      cTimer *timer = new cTimer;
      if (timer->Parse(Option)) {
-        cTimer *t = Timers.GetTimer(timer);
-        if (t) {
-           t->Parse(Option);
-           delete timer;
-           timer = t;
-           isyslog("timer %s updated", *timer->ToDescr());
+        if (!Timers.BeingEdited()) {
+           cTimer *t = Timers.GetTimer(timer);
+           if (t) {
+              t->Parse(Option);
+              delete timer;
+              timer = t;
+              isyslog("timer %s updated", *timer->ToDescr());
+              }
+           else {
+              Timers.Add(timer);
+              isyslog("timer %s added", *timer->ToDescr());
+              }
+           Timers.SetModified();
+           Reply(250, "%d %s", timer->Index() + 1, *timer->ToText());
+           return;
            }
-        else {
-           Timers.Add(timer);
-           isyslog("timer %s added", *timer->ToDescr());
-           }
-        Timers.SetModified();
-        Reply(250, "%d %s", timer->Index() + 1, *timer->ToText());
-        return;
+        else
+           Reply(550, "Timers are being edited - try again later");
         }
      else
         Reply(501, "Error in timer settings");
@@ -1125,6 +1383,7 @@ void cSVDRP::Execute(char *Cmd)
   else if (CMD("DELC"))  CmdDELC(s);
   else if (CMD("DELR"))  CmdDELR(s);
   else if (CMD("DELT"))  CmdDELT(s);
+  else if (CMD("EDIT"))  CmdEDIT(s);
   else if (CMD("GRAB"))  CmdGRAB(s);
   else if (CMD("HELP"))  CmdHELP(s);
   else if (CMD("HITK"))  CmdHITK(s);
@@ -1140,6 +1399,8 @@ void cSVDRP::Execute(char *Cmd)
   else if (CMD("NEWC"))  CmdNEWC(s);
   else if (CMD("NEWT"))  CmdNEWT(s);
   else if (CMD("NEXT"))  CmdNEXT(s);
+  else if (CMD("PLAY"))  CmdPLAY(s);
+  else if (CMD("PLUG"))  CmdPLUG(s);
   else if (CMD("PUTE"))  CmdPUTE(s);
   else if (CMD("SCAN"))  CmdSCAN(s);
   else if (CMD("STAT"))  CmdSTAT(s);
@@ -1177,6 +1438,11 @@ bool cSVDRP::Process(void)
                  // showtime!
                  Execute(cmdLine);
                  numChars = 0;
+                 if (length > BUFSIZ) {
+                    free(cmdLine); // let's not tie up too much memory
+                    length = BUFSIZ;
+                    cmdLine = MALLOC(char, length);
+                    }
                  }
               else if (c == 0x04 && numChars == 0) {
                  // end of file (only at beginning of line)
@@ -1190,15 +1456,14 @@ bool cSVDRP::Process(void)
               else if (c <= 0x03 || c == 0x0D) {
                  // ignore control characters
                  }
-              else if (numChars < sizeof(cmdLine) - 1) {
+              else {
+                 if (numChars >= length - 1) {
+                    length += BUFSIZ;
+                    cmdLine = (char *)realloc(cmdLine, length);
+                    }
                  cmdLine[numChars++] = c;
                  cmdLine[numChars] = 0;
                  }
-              else {
-                 Reply(501, "Command line too long");
-                 esyslog("SVDRP: command line too long: '%s'", cmdLine);
-                 numChars = 0;
-                 }
               lastActivity = time(NULL);
               }
            else if (r < 0) {
@@ -1217,11 +1482,4 @@ bool cSVDRP::Process(void)
   return false;
 }
 
-char *cSVDRP::GetMessage(void)
-{
-  char *s = message;
-  message = NULL;
-  return s;
-}
-
 //TODO more than one connection???
