@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: svdrp.h 1.20 2004/01/17 13:30:52 kls Exp $
+ * $Id: svdrp.h 1.26 2005/11/27 15:26:42 kls Exp $
  */
 
 #ifndef __SVDRP_H
@@ -45,18 +45,20 @@ private:
   cFile file;
   cRecordings Recordings;
   cPUTEhandler *PUTEhandler;
-  uint numChars;
-  char cmdLine[MAXPARSEBUFFER];
-  char *message;
+  int numChars;
+  int length;
+  char *cmdLine;
   time_t lastActivity;
   void Close(bool Timeout = false);
   bool Send(const char *s, int length = -1);
-  void Reply(int Code, const char *fmt, ...);
+  void Reply(int Code, const char *fmt, ...) __attribute__ ((format (printf, 3, 4)));
+  void PrintHelpTopics(const char **hp);
   void CmdCHAN(const char *Option);
   void CmdCLRE(const char *Option);
   void CmdDELC(const char *Option);
   void CmdDELR(const char *Option);
   void CmdDELT(const char *Option);
+  void CmdEDIT(const char *Option);
   void CmdGRAB(const char *Option);
   void CmdHELP(const char *Option);
   void CmdHITK(const char *Option);
@@ -72,6 +74,8 @@ private:
   void CmdNEWC(const char *Option);
   void CmdNEWT(const char *Option);
   void CmdNEXT(const char *Option);
+  void CmdPLAY(const char *Option);
+  void CmdPLUG(const char *Option);
   void CmdPUTE(const char *Option);
   void CmdSCAN(const char *Option);
   void CmdSTAT(const char *Option);
@@ -83,7 +87,6 @@ public:
   ~cSVDRP();
   bool HasConnection(void) { return file.IsOpen(); }
   bool Process(void);
-  char *GetMessage(void);
   };
 
 #endif //__SVDRP_H
