@@ -4,12 +4,13 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: timers.c 1.33 2005/06/18 12:49:55 kls Exp $
+ * $Id: timers.c 1.36 2005/09/09 15:22:33 kls Exp $
  */
 
 #include "timers.h"
 #include <ctype.h>
 #include "channels.h"
+#include "device.h"
 #include "i18n.h"
 #include "remote.h"
 
@@ -363,8 +364,6 @@ bool cTimer::Matches(time_t t, bool Directly) const
 
   if (HasFlags(tfActive)) {
      if (HasFlags(tfVps) && !Directly && event && event->Vps() && schedule && schedule->PresentSeenWithin(30)) {
-        if (startTime != event->Vps())
-           return false;
         startTime = event->StartTime();
         stopTime = event->EndTime();
         return event->IsRunning(true);
@@ -520,7 +519,10 @@ cTimers::cTimers(void)
 cTimer *cTimers::GetTimer(cTimer *Timer)
 {
   for (cTimer *ti = First(); ti; ti = Next(ti)) {
-      if (ti->Channel() == Timer->Channel() && ti->Day() == Timer->Day() && ti->Start() == Timer->Start() && ti->Stop() == Timer->Stop())
+      if (ti->Channel() == Timer->Channel() &&
+          (ti->WeekDays() && ti->WeekDays() == Timer->WeekDays() || !ti->WeekDays() && ti->Day() == Timer->Day()) &&
+          ti->Start() == Timer->Start() &&
+          ti->Stop() == Timer->Stop())
          return ti;
       }
   return NULL;
