@@ -4,18 +4,20 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: tools.c 1.95 2005/05/29 10:18:26 kls Exp $
+ * $Id: tools.c 1.104 2005/11/26 14:12:31 kls Exp $
  */
 
 #include "tools.h"
 #include <ctype.h>
 #include <dirent.h>
 #include <errno.h>
+#include <stdarg.h>
 #include <stdlib.h>
 #include <sys/time.h>
 #include <sys/vfs.h>
 #include <time.h>
 #include <unistd.h>
+#include <utime.h>
 #include "i18n.h"
 
 int SysLogLevel = 3;
@@ -308,7 +310,7 @@ bool MakeDirs(const char *FileName, bool IsDirectory)
         struct stat fs;
         if (stat(s, &fs) != 0 || !S_ISDIR(fs.st_mode)) {
            dsyslog("creating directory %s", s);
-           if (mkdir(s, S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH) == -1) {
+           if (mkdir(s, ACCESSPERMS) == -1) {
               LOG_ERROR_STR(s);
               result = false;
               break;
@@ -451,7 +453,7 @@ bool SpinUpDisk(const char *FileName)
       if (access(buf, F_OK) != 0) { // the file does not exist
          timeval tp1, tp2;
          gettimeofday(&tp1, NULL);
-         int f = open(buf, O_WRONLY | O_CREAT, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
+         int f = open(buf, O_WRONLY | O_CREAT, DEFFILEMODE);
          // O_SYNC doesn't work on all file systems
          if (f >= 0) {
             if (fdatasync(f) < 0)
@@ -461,7 +463,7 @@ bool SpinUpDisk(const char *FileName)
             gettimeofday(&tp2, NULL);
             double seconds = (((long long)tp2.tv_sec * 1000000 + tp2.tv_usec) - ((long long)tp1.tv_sec * 1000000 + tp1.tv_usec)) / 1000000.0;
             if (seconds > 0.5)
-               dsyslog("SpinUpDisk took %.2f seconds\n", seconds);
+               dsyslog("SpinUpDisk took %.2f seconds", seconds);
             free(buf);
             return true;
             }
@@ -474,6 +476,12 @@ bool SpinUpDisk(const char *FileName)
   return false;
 }
 
+void TouchFile(const char *FileName)
+{
+  if (utime(FileName, NULL) == -1 && errno != ENOENT)
+     LOG_ERROR_STR(FileName);
+}
+
 time_t LastModifiedTime(const char *FileName)
 {
   struct stat fs;
@@ -519,6 +527,11 @@ cString::cString(const char *S, bool TakePointer)
   s = TakePointer ? (char *)S : S ? strdup(S) : NULL;
 }
 
+cString::cString(const cString &String)
+{
+  s = String.s ? strdup(String.s) : NULL;
+}
+
 cString::~cString()
 {
   free(s);
@@ -526,10 +539,22 @@ cString::~cString()
 
 cString &cString::operator=(const cString &String)
 {
+  if (this == &String)
+     return *this;
+  free(s);
   s = String.s ? strdup(String.s) : NULL;
   return *this;
 }
 
+cString cString::sprintf(const char *fmt, ...)
+{
+  va_list ap;
+  va_start(ap, fmt);
+  char *buffer;
+  vasprintf(&buffer, fmt, ap);
+  return cString(buffer, true);
+}
+
 cString WeekDayName(int WeekDay)
 {
   char buffer[4];
@@ -592,12 +617,24 @@ cString TimeString(time_t t)
 
 // --- cReadLine -------------------------------------------------------------
 
+cReadLine::cReadLine(void)
+{
+  size = 0;
+  buffer = NULL;
+}
+
+cReadLine::~cReadLine()
+{
+  free(buffer);
+}
+
 char *cReadLine::Read(FILE *f)
 {
-  if (fgets(buffer, sizeof(buffer), f) > 0) {
-     int l = strlen(buffer) - 1;
-     if (l >= 0 && buffer[l] == '\n')
-        buffer[l] = 0;
+  int n = getline(&buffer, &size, f);
+  if (n > 0) {
+     n--;
+     if (buffer[n] == '\n')
+        buffer[n] = 0;
      return buffer;
      }
   return NULL;
@@ -819,6 +856,129 @@ bool cSafeFile::Close(void)
   return result;
 }
 
+// --- cUnbufferedFile -------------------------------------------------------
+
+#define READ_AHEAD MEGABYTE(2)
+#define WRITE_BUFFER MEGABYTE(10)
+
+cUnbufferedFile::cUnbufferedFile(void)
+{
+  fd = -1;
+}
+
+cUnbufferedFile::~cUnbufferedFile()
+{
+  Close();
+}
+
+int cUnbufferedFile::Open(const char *FileName, int Flags, mode_t Mode)
+{
+  Close();
+  fd = open(FileName, Flags, Mode);
+  begin = end = ahead = -1;
+  written = 0;
+  return fd;
+}
+
+int cUnbufferedFile::Close(void)
+{
+  if (fd >= 0) {
+     if (ahead > end)
+        end = ahead;
+     if (begin >= 0 && end > begin) {
+        //dsyslog("close buffer: %d (flush: %d bytes, %ld-%ld)", fd, written, begin, end);
+        if (written)
+           fdatasync(fd);
+        posix_fadvise(fd, begin, end - begin, POSIX_FADV_DONTNEED);
+        }
+     begin = end = ahead = -1;
+     written = 0;
+     }
+  int OldFd = fd;
+  fd = -1;
+  return close(OldFd);
+}
+
+off_t cUnbufferedFile::Seek(off_t Offset, int Whence)
+{
+  if (fd >= 0)
+     return lseek(fd, Offset, Whence);
+  return -1;
+}
+
+ssize_t cUnbufferedFile::Read(void *Data, size_t Size)
+{
+  if (fd >= 0) {
+     off_t pos = lseek(fd, 0, SEEK_CUR);
+     // jump forward - adjust end position
+     if (pos > end)
+        end = pos;
+     // after adjusting end - don't clear more than previously requested
+     if (end > ahead)
+        end = ahead;
+     // jump backward - drop read ahead of previous run
+     if (pos < begin)
+        end = ahead;
+     if (begin >= 0 && end > begin)
+        posix_fadvise(fd, begin - KILOBYTE(200), end - begin + KILOBYTE(200), POSIX_FADV_DONTNEED);//XXX macros/parameters???
+     begin = pos;
+     ssize_t bytesRead = safe_read(fd, Data, Size);
+     if (bytesRead > 0) {
+        pos += bytesRead;
+        end = pos;
+        // this seems to trigger a non blocking read - this
+        // may or may not have been finished when we will be called next time.
+        // If it is not finished we can't release the not yet filled buffers.
+        // So this is commented out till we find a better solution.
+        //posix_fadvise(fd, pos, READ_AHEAD, POSIX_FADV_WILLNEED);
+        ahead = pos + READ_AHEAD;
+        }
+     else
+        end = pos;
+     return bytesRead;
+     }
+  return -1;
+}
+
+ssize_t cUnbufferedFile::Write(const void *Data, size_t Size)
+{
+  if (fd >=0) {
+     off_t pos = lseek(fd, 0, SEEK_CUR);
+     ssize_t bytesWritten = safe_write(fd, Data, Size);
+     if (bytesWritten >= 0) {
+        written += bytesWritten;
+        if (begin >= 0) {
+           if (pos < begin)
+              begin = pos;
+           }
+        else
+           begin = pos;
+        if (pos + bytesWritten > end)
+           end = pos + bytesWritten;
+        if (written > WRITE_BUFFER) {
+           //dsyslog("flush buffer: %d (%d bytes, %ld-%ld)", fd, written, begin, end);
+           fdatasync(fd);
+           if (begin >= 0 && end > begin)
+              posix_fadvise(fd, begin, end - begin, POSIX_FADV_DONTNEED);
+           begin = end = -1;
+           written = 0;
+           }
+        }
+     return bytesWritten;
+     }
+  return -1;
+}
+
+cUnbufferedFile *cUnbufferedFile::Create(const char *FileName, int Flags, mode_t Mode)
+{
+  cUnbufferedFile *File = new cUnbufferedFile;
+  if (File->Open(FileName, Flags, Mode) < 0) {
+     delete File;
+     File = NULL;
+     }
+  return File;
+}
+
 // --- cLockFile -------------------------------------------------------------
 
 #define LOCKFILENAME      ".lock-vdr"
@@ -843,7 +1003,7 @@ bool cLockFile::Lock(int WaitSeconds)
   if (f < 0 && fileName) {
      time_t Timeout = time(NULL) + WaitSeconds;
      do {
-        f = open(fileName, O_WRONLY | O_CREAT | O_EXCL, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
+        f = open(fileName, O_WRONLY | O_CREAT | O_EXCL, DEFFILEMODE);
         if (f < 0) {
            if (errno == EEXIST) {
               struct stat fs;
@@ -1070,8 +1230,7 @@ cHashBase::cHashBase(int Size)
 
 cHashBase::~cHashBase(void)
 {
-  for (int i = 0; i < size; i++)
-      delete hashTable[i];
+  Clear();
   free(hashTable);
 }
 
@@ -1096,6 +1255,14 @@ void cHashBase::Del(cListObject *Object, unsigned int Id)
      }
 }
 
+void cHashBase::Clear(void)
+{
+  for (int i = 0; i < size; i++) {
+      delete hashTable[i];
+      hashTable[i] = NULL;
+      }
+}
+
 cListObject *cHashBase::Get(unsigned int Id) const
 {
   cList<cHashObject> *list = hashTable[hashfn(Id)];
