@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: tools.h 1.72 2005/05/29 10:24:54 kls Exp $
+ * $Id: tools.h 1.84 2005/11/26 14:03:47 kls Exp $
  */
 
 #ifndef __TOOLS_H
@@ -38,14 +38,12 @@ extern int SysLogLevel;
 #define KILOBYTE(n) ((n) * 1024)
 #define MEGABYTE(n) ((n) * 1024 * 1024)
 
-#define MAXPARSEBUFFER KILOBYTE(10)
-
 #define MALLOC(type, size)  (type *)malloc(sizeof(type) * (size))
 
 #define DELETENULL(p) (delete (p), p = NULL)
 
 #define CHECK(s) { if ((s) < 0) LOG_ERROR; } // used for 'ioctl()' calls
-#define FATALERRNO (errno != EAGAIN && errno != EINTR)
+#define FATALERRNO (errno && errno != EAGAIN && errno != EINTR)
 
 #ifndef __STL_CONFIG_H // in case some plugin needs to use the STL
 template<class T> inline T min(T a, T b) { return a <= b ? a : b; }
@@ -57,15 +55,32 @@ template<class T> inline void swap(T &a, T &b) { T t = a; a = b; b = t; }
 #define BCDCHARTOINT(x) (10 * ((x & 0xF0) >> 4) + (x & 0xF))
 int BCD2INT(int x);
 
+// Unfortunately there are no platform independent macros for unaligned
+// access. so we do it this way:
+
+template<class T> inline T get_unaligned(T *p)
+{
+  struct s { T v; } __attribute__((packed));
+  return ((s *)p)->v;
+}
+
+template<class T> inline void put_unaligned(unsigned int v, T* p)
+{
+  struct s { T v; } __attribute__((packed));
+  ((s *)p)->v = v;
+}
+
 class cString {
 private:
   char *s;
 public:
   cString(const char *S = NULL, bool TakePointer = false);
+  cString(const cString &String);
   virtual ~cString();
   operator const char * () const { return s; } // for use in (const char *) context
   const char * operator*() const { return s; } // for use in (const void *) context (printf() etc.)
   cString &operator=(const cString &String);
+  static cString sprintf(const char *fmt, ...) __attribute__ ((format (printf, 1, 2)));
   };
 
 ssize_t safe_read(int filedes, void *buffer, size_t size);
@@ -95,8 +110,9 @@ bool DirectoryOk(const char *DirName, bool LogErrors = false);
 bool MakeDirs(const char *FileName, bool IsDirectory = false);
 bool RemoveFileOrDir(const char *FileName, bool FollowSymlinks = false);
 bool RemoveEmptyDirectories(const char *DirName, bool RemoveThis = false);
-char *ReadLink(const char *FileName); ///< returns a new strings allocated on the heap, which the caller must delete (or NULL in case of an error)
+char *ReadLink(const char *FileName); ///< returns a new string allocated on the heap, which the caller must delete (or NULL in case of an error)
 bool SpinUpDisk(const char *FileName);
+void TouchFile(const char *FileName);
 time_t LastModifiedTime(const char *FileName);
 cString WeekDayName(int WeekDay);
 cString WeekDayName(time_t t);
@@ -118,8 +134,11 @@ public:
 
 class cReadLine {
 private:
-  char buffer[MAXPARSEBUFFER];
+  size_t size;
+  char *buffer;
 public:
+  cReadLine(void);
+  ~cReadLine();
   char *Read(FILE *f);
   };
 
@@ -158,7 +177,7 @@ public:
   cFile(void);
   ~cFile();
   operator int () { return f; }
-  bool Open(const char *FileName, int Flags, mode_t Mode = S_IRUSR | S_IWUSR | S_IRGRP);
+  bool Open(const char *FileName, int Flags, mode_t Mode = DEFFILEMODE);
   bool Open(int FileDes);
   void Close(void);
   bool IsOpen(void) { return f >= 0; }
@@ -181,6 +200,27 @@ public:
   bool Close(void);
   };
 
+/// cUnbufferedFile is used for large files that are mainly written or read
+/// in a streaming manner, and thus should not be cached.
+
+class cUnbufferedFile {
+private:
+  int fd;
+  off_t begin;
+  off_t end;
+  off_t ahead;
+  ssize_t written;
+public:
+  cUnbufferedFile(void);
+  ~cUnbufferedFile();
+  int Open(const char *FileName, int Flags, mode_t Mode = DEFFILEMODE);
+  int Close(void);
+  off_t Seek(off_t Offset, int Whence);
+  ssize_t Read(void *Data, size_t Size);
+  ssize_t Write(const void *Data, size_t Size);
+  static cUnbufferedFile *Create(const char *FileName, int Flags, mode_t Mode = DEFFILEMODE);
+  };
+
 class cLockFile {
 private:
   char *fileName;
@@ -243,6 +283,7 @@ private:
   cListObject *object;
 public:
   cHashObject(cListObject *Object, unsigned int Id) { object = Object; id = Id; }
+  cListObject *Object(void) { return object; }
   };
 
 class cHashBase {
@@ -256,6 +297,7 @@ public:
   virtual ~cHashBase();
   void Add(cListObject *Object, unsigned int Id);
   void Del(cListObject *Object, unsigned int Id);
+  void Clear(void);
   cListObject *Get(unsigned int Id) const;
   cList<cHashObject> *GetList(unsigned int Id) const;
   };
