@@ -22,7 +22,7 @@
  *
  * The project's page is at http://www.cadsoft.de/vdr
  *
- * $Id: vdr.c 1.208 2005/06/18 11:19:07 kls Exp $
+ * $Id: vdr.c 1.220 2005/11/27 15:56:18 kls Exp $
  */
 
 #include <getopt.h>
@@ -63,6 +63,10 @@
 #define SHUTDOWNWAIT      300 // seconds to wait in user prompt before automatic shutdown
 #define MANUALSTART       600 // seconds the next timer must be in the future to assume manual start
 #define CHANNELSAVEDELTA  600 // seconds before saving channels.conf after automatic modifications
+#define LASTCAMMENUTIMEOUT  3 // seconds to run the main loop 'fast' after a CAM menu has been closed
+                              // in order to react on a possible new CAM menu as soon as possible
+#define DEVICEREADYTIMEOUT 30 // seconds to wait until all devices are ready
+#define MENUTIMEOUT       120 // seconds of user inactivity after which an OSD display is closed
 
 #define EXIT(v) { ExitCode = (v); goto Exit; }
 
@@ -115,6 +119,22 @@ int main(int argc, char *argv[])
   int WatchdogTimeout = DEFAULTWATCHDOG;
   const char *Terminal = NULL;
   const char *Shutdown = NULL;
+
+  bool UseKbd = true;
+  const char *LircDevice = NULL;
+  const char *RcuDevice = NULL;
+#if !defined(REMOTE_KBD)
+  UseKbd = false;
+#endif
+#if defined(REMOTE_LIRC)
+  LircDevice = LIRC_DEVICE;
+#elif defined(REMOTE_RCU)
+  RcuDevice = RCU_DEVICE;
+#endif
+#if defined(VFAT)
+  VfatFileSystem = true;
+#endif
+
   cPluginManager PluginManager(DEFAULTPLUGINDIR);
   int ExitCode = 0;
 
@@ -126,14 +146,18 @@ int main(int argc, char *argv[])
       { "epgfile",  required_argument, NULL, 'E' },
       { "help",     no_argument,       NULL, 'h' },
       { "lib",      required_argument, NULL, 'L' },
+      { "lirc",     optional_argument, NULL, 'l' | 0x100 },
       { "log",      required_argument, NULL, 'l' },
       { "mute",     no_argument,       NULL, 'm' },
+      { "no-kbd",   no_argument,       NULL, 'n' | 0x100 },
       { "plugin",   required_argument, NULL, 'P' },
       { "port",     required_argument, NULL, 'p' },
+      { "rcu",      optional_argument, NULL, 'r' | 0x100 },
       { "record",   required_argument, NULL, 'r' },
       { "shutdown", required_argument, NULL, 's' },
       { "terminal", required_argument, NULL, 't' },
       { "version",  no_argument,       NULL, 'V' },
+      { "vfat",     no_argument,       NULL, 'v' | 0x100 },
       { "video",    required_argument, NULL, 'v' },
       { "watchdog", required_argument, NULL, 'w' },
       { NULL }
@@ -194,8 +218,14 @@ int main(int argc, char *argv[])
                        return 2;
                        }
                     break;
+          case 'l' | 0x100:
+                    LircDevice = optarg ? : LIRC_DEVICE;
+                    break;
           case 'm': MuteAudio = true;
                     break;
+          case 'n' | 0x100:
+                    UseKbd = false;
+                    break;
           case 'p': if (isnumber(optarg))
                        SVDRPport = atoi(optarg);
                     else {
@@ -205,6 +235,9 @@ int main(int argc, char *argv[])
                     break;
           case 'P': PluginManager.AddPlugin(optarg);
                     break;
+          case 'r' | 0x100:
+                    RcuDevice = optarg ? : RCU_DEVICE;
+                    break;
           case 'r': cRecordingUserCommand::SetCommand(optarg);
                     break;
           case 's': Shutdown = optarg;
@@ -217,6 +250,9 @@ int main(int argc, char *argv[])
                     break;
           case 'V': DisplayVersion = true;
                     break;
+          case 'v' | 0x100:
+                    VfatFileSystem = true;
+                    break;
           case 'v': VideoDirectory = optarg;
                     while (optarg && *optarg && optarg[strlen(optarg) - 1] == '/')
                           optarg[strlen(optarg) - 1] = 0;
@@ -261,21 +297,30 @@ int main(int argc, char *argv[])
                "                           if logging should be done to LOG_LOCALn instead of\n"
                "                           LOG_USER, add '.n' to LEVEL, as in 3.7 (n=0..7)\n"
                "  -L DIR,   --lib=DIR      search for plugins in DIR (default is %s)\n"
+               "            --lirc[=PATH]  use a LIRC remote control device, attached to PATH\n"
+               "                           (default: %s)\n"
                "  -m,       --mute         mute audio of the primary DVB device at startup\n"
+               "            --no-kbd       don't use the keyboard as an input device\n"
                "  -p PORT,  --port=PORT    use PORT for SVDRP (default: %d)\n"
                "                           0 turns off SVDRP\n"
                "  -P OPT,   --plugin=OPT   load a plugin defined by the given options\n"
+               "            --rcu[=PATH]   use a remote control device, attached to PATH\n"
+               "                           (default: %s)\n"
                "  -r CMD,   --record=CMD   call CMD before and after a recording\n"
                "  -s CMD,   --shutdown=CMD call CMD to shutdown the computer\n"
                "  -t TTY,   --terminal=TTY controlling tty\n"
                "  -v DIR,   --video=DIR    use DIR as video directory (default: %s)\n"
                "  -V,       --version      print version information and exit\n"
+               "            --vfat         encode special characters in recording names to\n"
+               "                           avoid problems with VFAT file systems\n"
                "  -w SEC,   --watchdog=SEC activate the watchdog timer with a timeout of SEC\n"
                "                           seconds (default: %d); '0' disables the watchdog\n"
                "\n",
                DEFAULTEPGDATAFILENAME,
                DEFAULTPLUGINDIR,
+               LIRC_DEVICE,
                DEFAULTSVDRPPORT,
+               RCU_DEVICE,
                VideoDirectory,
                DEFAULTWATCHDOG
                );
@@ -325,17 +370,11 @@ int main(int argc, char *argv[])
   // Daemon mode:
 
   if (DaemonMode) {
-     pid_t pid = fork();
-     if (pid < 0) {
+     if (daemon(1, 0) == -1) {
         fprintf(stderr, "%m\n");
         esyslog("ERROR: %m");
         return 2;
         }
-     if (pid != 0)
-        return 0; // initial program immediately returns
-     fclose(stdin);
-     fclose(stdout);
-     fclose(stderr);
      }
   else if (Terminal) {
      // Claim new controlling terminal
@@ -357,6 +396,7 @@ int main(int argc, char *argv[])
   int PreviousChannelIndex = 0;
   time_t LastChannelChanged = time(NULL);
   time_t LastActivity = 0;
+  time_t LastCamMenu = 0;
   int MaxLatencyTime = 0;
   bool ForceShutdown = false;
   bool UserShutdown = false;
@@ -391,6 +431,10 @@ int main(int argc, char *argv[])
 
   cFont::SetCode(I18nCharSets()[Setup.OSDLanguage]);
 
+  // Recordings:
+
+  Recordings.Update();
+
   // EPG data:
 
   if (EpgDataFileName) {
@@ -470,15 +514,12 @@ int main(int argc, char *argv[])
   cThemes::Load(Skins.Current()->Name(), Setup.OSDTheme, Skins.Current()->Theme());
 
   // Remote Controls:
-#if defined(REMOTE_RCU)
-  new cRcuRemote("/dev/ttyS1");
-#elif defined(REMOTE_LIRC)
-  new cLircRemote("/dev/lircd");
-#endif
-#if defined(REMOTE_KBD)
-  if (!DaemonMode && HasStdin)
+  if (RcuDevice)
+     new cRcuRemote(RcuDevice);
+  if (LircDevice)
+     new cLircRemote(LircDevice);
+  if (!DaemonMode && HasStdin && UseKbd)
      new cKbdRemote;
-#endif
   Interface->LearnKeys();
 
   // External audio:
@@ -488,16 +529,14 @@ int main(int argc, char *argv[])
 
   // Channel:
 
+  if (!cDevice::WaitForAllDevicesReady(DEVICEREADYTIMEOUT))
+     dsyslog("not all devices ready after %d seconds", DEVICEREADYTIMEOUT);
   Channels.SwitchTo(Setup.CurrentChannel);
   if (MuteAudio)
      cDevice::PrimaryDevice()->ToggleMute();
   else
      cDevice::PrimaryDevice()->SetVolume(Setup.CurrentVolume, true);
 
-  // Recordings:
-
-  Recordings.Load();
-
   // Signal handlers:
 
   if (signal(SIGHUP,  SignalHandler) == SIG_IGN) signal(SIGHUP,  SIG_IGN);
@@ -629,13 +668,21 @@ int main(int argc, char *argv[])
                }
            }
         if (!Menu && Recordings.NeedsUpdate())
-           Recordings.Load();
+           Recordings.Update();
         // CAM control:
-        if (!Menu && !cOsd::IsOpen())
+        if (!Menu && !cOsd::IsOpen()) {
            Menu = CamControl();
+           if (Menu)
+              LastCamMenu = 0;
+           else if (!LastCamMenu)
+              LastCamMenu = time(NULL);
+           }
+        // Queued messages:
+        if (!Skins.IsOpen())
+           Skins.ProcessQueuedMessages();
         // User Input:
         cOsdObject *Interact = Menu ? Menu : cControl::Control();
-        eKeys key = Interface->GetKey(!Interact || !Interact->NeedsFastResponse());
+        eKeys key = Interface->GetKey((!Interact || !Interact->NeedsFastResponse()) && time(NULL) - LastCamMenu > LASTCAMMENUTIMEOUT);
         if (NORMALKEY(key) != kNone) {
            EITScanner.Activity();
            LastActivity = time(NULL);
@@ -655,7 +702,7 @@ int main(int argc, char *argv[])
                Menu = new cMenuMain(cControl::Control());
                Temp = NULL;
                break;
-          #define DirectMainFunction(function...)\
+          #define DirectMainFunction(function)\
             DELETENULL(Menu);\
             if (cControl::Control())\
                cControl::Control()->Hide();\
@@ -669,7 +716,25 @@ int main(int argc, char *argv[])
           case kSetup:      DirectMainFunction(osSetup); break;
           case kCommands:   DirectMainFunction(osCommands); break;
           case kUser1 ... kUser9: cRemote::PutMacro(key); key = kNone; break;
-          case k_Plugin:    DirectMainFunction(osPlugin, cRemote::GetPlugin()); break;
+          case k_Plugin: {
+               DELETENULL(Menu);
+               Temp = NULL;
+               if (cControl::Control())
+                  cControl::Control()->Hide();
+               cPlugin *plugin = cPluginManager::GetPlugin(cRemote::GetPlugin());
+               if (plugin) {
+                  Menu = Temp = plugin->MainMenuAction();
+                  if (Menu) {
+                     Menu->Show();
+                     if (Menu->IsMenu())
+                        ((cOsdMenu*)Menu)->Display();
+                     }
+                  }
+               else
+                  esyslog("ERROR: unknown plugin '%s'", cRemote::GetPlugin());
+               key = kNone; // nobody else needs to see these keys
+               }
+               break;
           // Channel up/down:
           case kChanUp|k_Repeat:
           case kChanUp:
@@ -750,8 +815,18 @@ int main(int argc, char *argv[])
         Interact = Menu ? Menu : cControl::Control(); // might have been closed in the mean time
         if (Interact) {
            eOSState state = Interact->ProcessKey(key);
-           if (state == osUnknown && ISMODELESSKEY(key) && cControl::Control() && Interact != cControl::Control())
-              state = cControl::Control()->ProcessKey(key);
+           if (state == osUnknown && Interact != cControl::Control()) {
+              if (ISMODELESSKEY(key) && cControl::Control()) {
+                 state = cControl::Control()->ProcessKey(key);
+                 if (state == osEnd) {
+                    // let's not close a menu when replay ends:
+                    cControl::Shutdown();
+                    continue;
+                    }
+                 }
+              else if (time(NULL) - LastActivity > MENUTIMEOUT)
+                 state = osEnd;
+              }
            switch (state) {
              case osPause:  DELETENULL(Menu);
                             cControl::Shutdown(); // just in case
